﻿using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

/// <summary>
/// Summary description for Base Event Item
/// </summary>
public abstract class BaseEventItem<T> : UserControl, IBaseEventItem, IDynamicControl
    where T : BaseEventModel
{
    private T _eventViewModel;

    private string _validationGroup;

    #region Protected Shared Members

    protected T EventViewModel
    {
        get { return ViewState["ViewModel"] as T; }
        set { ViewState["ViewModel"] = value; }
    }

    protected string ValidationGroup
    {
        get { return ViewState["EventValidationGroup"] as string; }
        set { ViewState["EventValidationGroup"] = value; }
    }

    protected bool ForceRefresh;

    #endregion

    #region Abstract Methods

    protected abstract T SaveItem();

    protected abstract void MapValidationGroup();

    protected abstract void MapTestSummary();

    protected abstract void MapTestDetails();

    protected abstract void PerformForcedRefresh();

    #endregion

    #region Shared Page Methods

    protected override void OnLoad(EventArgs e)
    {
        base.OnLoad(e);

        if (ValidationGroup == null || ForceRefresh)
        {
            ValidationGroup = _validationGroup;
        }

        if (EventViewModel == null || ForceRefresh)
        {
            // Initialize the view model state
            EventViewModel = _eventViewModel;
            ForceRefresh = false;
        }

        if (EventViewModel != null)
        {
            MapTestSummary();
            MapTestDetails();
        }

        if (!string.IsNullOrEmpty(ValidationGroup))
        {
            MapValidationGroup();
        }
    }

    #endregion

    #region Implemented Methods

    void IDynamicControl.PerformForcedRefresh()
    {
        PerformForcedRefresh();
    }

    void IBaseEventItem.SetValidationGroup(string validationGroup)
    {
        _validationGroup = validationGroup;
    }

    void IBaseEventItem.SetViewModel(BaseEventModel viewModel)
    {
        T derivedViewModel = viewModel as T;

        if (derivedViewModel != null)
        {
            _eventViewModel = derivedViewModel;
            ForceRefresh = true;
            PerformForcedRefresh();
        }
    }

    BaseEventModel IBaseEventItem.SaveItem()
    {
        EventViewModel = SaveItem();
        return EventViewModel;
    }

    #endregion

    #region Shared Methods

    protected virtual void MapSelectField(RadComboBox comboBox, SingleSelectQuestion singleSelectQuestion)
    {
        comboBox.MarkFirstMatch = true;
        comboBox.IsCaseSensitive = false;
        comboBox.OnClientBlur = "MEVIR.Common.RadComboBoxValidateText";
        comboBox.OnClientTemplateDataBound = "MEVIR.Common.RadComboBoxValidateText";
        comboBox.ToolTip = string.Format("Select The {0}", singleSelectQuestion.DisplayText);
        comboBox.DataSource = singleSelectQuestion.AllChoicesOrdered;
        comboBox.DataValueField = "Code";
        comboBox.DataTextField = "DisplayText";
        comboBox.DataBind();

        if (singleSelectQuestion.IsAnswered)
        {
            comboBox.SelectedValue = singleSelectQuestion.SelectedChoice.Code;
        }
    }

    protected virtual void MapSelectField(RadComboBox comboBox, TextBox textBox, SingleSelectQuestion singleSelectQuestion, bool isReadOnly)
    {
        MapSelectField(comboBox, singleSelectQuestion);

        if (isReadOnly)
        {
            comboBox.CssClass += " displayNone";
            textBox.ToolTip = string.Format("Read Only - {0}", singleSelectQuestion.DisplayText);
            textBox.Visible = true;
            textBox.Text = comboBox.Text;
        }
    }

    protected virtual void MapSelectField(RadComboBox comboBox, MultiSelectQuestion multiSelectQuestion)
    {
        comboBox.MarkFirstMatch = true;
        comboBox.IsCaseSensitive = false;
        comboBox.OnClientBlur = "MEVIR.Common.RadComboBoxValidateText";
        comboBox.OnClientTemplateDataBound = "MEVIR.Common.RadComboBoxValidateText";
        comboBox.ToolTip = string.Format("Select The {0}", multiSelectQuestion.DisplayText);
        comboBox.DataSource = multiSelectQuestion.AvailableChoicesOrdered;
        comboBox.DataValueField = "Code";
        comboBox.DataTextField = "DisplayText";
        comboBox.DataBind();
    }

    protected virtual void MapMultiSelectGrid(RadGrid grid, MultiSelectQuestion multiSelectQuestion, bool isReadOnly)
    {
        grid.MasterTableView.Caption = multiSelectQuestion.DisplayText;
        grid.MasterTableView.Columns[0].HeaderText = multiSelectQuestion.DisplayText;
        MapMultiSelectGrid(grid, multiSelectQuestion.SelectedChoicesOrderd, isReadOnly);
    }

    protected virtual void MapMultiSelectGrid(RadGrid grid, IEnumerable<object> viewList, bool isReadOnly)
    {
        if (isReadOnly)
        {
            int editColumnNum = grid.MasterTableView.Columns.Count - 1;
            grid.MasterTableView.Columns[editColumnNum].Visible = false;
            grid.MasterTableView.ShowFooter = false;
        }

        grid.DataSource = viewList;
        grid.DataBind();
    }

    protected virtual void MapTextField(TextBox textBox, TextQuestion textQuestion, bool isReadOnly)
    {
        textBox.ToolTip = string.Format("Input The {0}", textQuestion.DisplayText);
        textBox.Text = textQuestion.ResponseText;
        textBox.ReadOnly = isReadOnly;
    }

    protected virtual void MapTextField(RadMaskedTextBox textBox, TextQuestion textQuestion, bool isReadOnly)
    {
        textBox.ToolTip = string.Format("Input The {0}", textQuestion.DisplayText);
        textBox.Text = textQuestion.ResponseText;
        textBox.ReadOnly = isReadOnly;
        textBox.EmptyMessage = isReadOnly ? string.Empty : textBox.EmptyMessage;
    }

    protected virtual void MapTextField(RadTextBox textBox, TextQuestion textQuestion, bool isReadOnly)
    {
        textBox.ToolTip = string.Format("Input The {0}", textQuestion.DisplayText);
        textBox.Text = textQuestion.ResponseText;
        textBox.ReadOnly = isReadOnly;
        textBox.EmptyMessage = isReadOnly ? string.Empty : textBox.EmptyMessage;
    }
    
    protected string GetNumberResponse(string response)
    {
        // Either return a number or empty string
        int size;
        return int.TryParse(response.Trim(), out size) ? size.ToString() : string.Empty;
    }

    protected string GetTextResponse(RadMaskedTextBox textBox)
    {
        return !string.IsNullOrEmpty(textBox.Text) ? textBox.TextWithLiterals : string.Empty;
    }

    #endregion

}