﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic;
using System.Text;
using System.Data;
using System.Data.Linq;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;

namespace Legacy.DVEIR
{
    public enum DVEIRReferralView
    {
        All,
        Encounters,
        Patient,
        ProblemList,
        ReviewComments,
        Provider,
        ChiefComplaint,
        Exam,
        Diagnosis,
        Procedures

    }

    [Serializable()]
    [System.ComponentModel.DataObject]
    public class DVEIRReferralManager : BaseBO
    {
        private UserAccountManager _user = (UserAccountManager)System.Threading.Thread.CurrentPrincipal;
        private RegistriesCommonManager _commonManager = new RegistriesCommonManager();
        private DVEIRCommonManager _commonManagerDveir = new DVEIRCommonManager();

        public enum TimePeriodForDisplayingCompletedRecords
        {
            Days = 60
        }

        public bool SaveItem(DVEIR_REFERRAL referral, PATIENT patient)
        {
            bool saveError = false;

            int returnStatus = 0;

            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                patient.SynchroniseWithDataContext(_db);
                referral.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.

                int status = referral.EnforceBusinessRules(true, _db);
                
                if (referral.BrokenRules.Count == 0)
                {

                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();
                        //commenting out this section so that concurrent saves on encounters in same referral 
                        //do not cause a linq exception
                        //referral.UPDATEDBY = "ItemUpdate";

                        //call submit changes if changes exist
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (Exception e)
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }


            if (returnStatus < 0)
            {
                saveError = true;
            }

            return !saveError;
        }


        public DVEIRReferralManager() 
        {
            _defaultSortField = "REFERRAL_ID";

        }

        #region Methods for Set Load
        private IQueryable<DVEIR_REFERRAL> LinqAll()
        {
           

            //populate LinqAll
            //Referral linq class has a subtype of DVEIR_REFERRAL.  The Registry association is specified in the dbml.
            //Therefore adding the OfType in the following query will only return TBI_REFERRAL objects.
            IQueryable<DVEIR_REFERRAL> tempLinq = (from e in _db.REFERRALs.OfType<DVEIR_REFERRAL>()
                                                 select e);

                 
            return tempLinq;

        }
        private void SetLoadWith(RegistriesDataAccess db, DVEIRReferralView view)
        {
            DataLoadOptions lo = new DataLoadOptions();
            
            lo.LoadWith<REFERRAL>(e => e.STD_REFERRALST);
            lo.LoadWith<REFERRAL>(e => e.ER_REFERRAL_REVIEWCOMMENTs);

            lo.LoadWith<REFERRAL>(e => e.ER_ENCOUNTERs);
            lo.AssociateWith<REFERRAL>
                (e => e.ER_ENCOUNTERs
                            .Where(
                                    p => 
                                        //show all encounters when in completed status
                                        p.REFERRAL.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_COMPLETED
                                        ||
                                        //show non sent encounters when in ready to send status
                                        (p.REFERRAL.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR
                                          &&
                                          (p.SendTo_DoD_DVEIR_FLAG == null
                                                || p.SendTo_DoD_DVEIR_FLAG == false)
                                        )
                                        || 
                                        //for all other statuses only show encounters that are not marked ready and not sent
                                        (
                                         (p.Ready_to_Send_to_DVEIR_FLAG == null
                                                ||  p.Ready_to_Send_to_DVEIR_FLAG == false)
                                         && 
                                         (p.SendTo_DoD_DVEIR_FLAG == null
                                                || p.SendTo_DoD_DVEIR_FLAG == false)
                                        )  
                                  )
                );

            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_ENCOUNTER_REVIEWCOMMENTs);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;


        }

        private void SetLoadWithOneEncounter(RegistriesDataAccess db, int encounterId, DVEIRReferralView view)
        {
            DataLoadOptions lo = new DataLoadOptions();
            
            lo.LoadWith<REFERRAL>(e => e.STD_REFERRALST);
            lo.LoadWith<REFERRAL>(e => e.STD_REFERRALRANK);
            lo.LoadWith<REFERRAL>(e => e.ER_ENCOUNTERs);
            lo.AssociateWith<REFERRAL>(e => e.ER_ENCOUNTERs.Where(p => p.ER_ENCOUNTER_ID == encounterId));
            lo.LoadWith<ER_ENCOUNTER>(e => e.PROVIDER);
            lo.LoadWith<PROVIDER>(e => e.PROVIDER_CLASS_DTLs);
            lo.LoadWith<PROVIDER_CLASS_DTL>(e => e.STD_PROVIDER_CLASS);
                
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_ENCOUNTER_REVIEWCOMMENTs);

            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_EXAM);

            lo.LoadWith<ER_EXAM>(e => e.ER_ACUITY_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_EXAM_DTLs);
            lo.LoadWith<ER_ACUITY_EXAM_DTL>(e => e.STD_ER_ACUITY_TYPE);
            lo.LoadWith<ER_EXAM>(e => e.ER_ANTERIOR_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_BROW_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_CORNEA_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_DISC_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_GLOBE_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_IMAGING_DTLs);
            lo.LoadWith<ER_IMAGING_DTL>(e => e.STD_ER_IMG_MTHD);
            lo.LoadWith<ER_EXAM>(e => e.ER_IRIS_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_ANGLE_RECESSION_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_LENS_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_LID_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_MISC_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_MOTILITY_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_ORBIT_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_PUPIL_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_RAPD_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_RETINA_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_SCLERA_EXAM_DTLs);
            lo.LoadWith<ER_EXAM>(e => e.ER_VITREOUS_EXAM_DTLs);
    
            lo.LoadWith<ER_ENCOUNTER>(e => e.ER_OUTCOME);
            lo.LoadWith<ER_OUTCOME>(e => e.ER_PROSTHETIC_DTLs);
            lo.LoadWith<ER_PROSTHETIC_DTL>(e => e.STD_ER_PROSTHETIC_TYPE);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }


        private void SetLoadWithForListViews(RegistriesDataAccess db)
        {
            DataLoadOptions lo = new DataLoadOptions();
            lo.LoadWith<REFERRAL>(e => e.STD_REFERRALRANK);
            lo.LoadWith<REFERRAL>(e => e.PATIENT);
            lo.LoadWith<REFERRAL>(e => e.STD_INSTITUTION);
            lo.LoadWith<REFERRAL>(e => e.STD_REFERRALST);
            lo.LoadWith<STD_INSTITUTION>(e => e.STD_INSTITUTION_PARENT);

            lo.LoadWith<REFERRAL>(e => e.WKF_CASEs);
            lo.LoadWith<WKF_CASE>(e => e.WKF_CASE_ACTIVITies);

                lo.AssociateWith<WKF_CASE>(
                    e => e.WKF_CASE_ACTIVITies.OrderByDescending(p => p.WKF_CASE_ACTIVITY_ID).Take(1));
                lo.LoadWith<WKF_CASE_ACTIVITY>(e => e.STD_WKFACTIVITYTYPE);

            db.LoadOptions = lo;
            db.DeferredLoadingEnabled = false;

        }
        #endregion Methods for Set Load

        #region Method for Check Existence
        public bool ReferralExists(int id)
        {
            using (_db = GetDataContext())
            {
                int count = (from e in _db.REFERRALs where e.REFERRAL_ID == id select e).Count();
                
                if (count > 0)
                {
                    return true;
                }
                else 
                {
                    return false;
                }
            }
        }
        #endregion

        #region Methods for Select

        public IEnumerable<DVEIR_REFERRAL> Select(string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                List<DVEIR_REFERRAL> entities;
                entities = SelectLinqFilter().OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
                 
            }
        }
        public int SelectCount(string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectLinqFilter().Count();
            }
        }
        private IQueryable<DVEIR_REFERRAL> SelectLinqFilter()
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();
            return linqFilter;
        }

        #endregion

        #region Methods for SelectByID

        public DVEIR_REFERRAL SelectByID(int id, DVEIRReferralView view)
        {
            bool refresh = true;

            if (DVEIRGlobal.CurrentDveivrReferral != null)
            {
                if (DVEIRGlobal.CurrentDveivrReferral.REFERRAL_ID == id)
                {
                    refresh = false;
                }
            }

            if (refresh)
            {
                using (_db = GetDataContext())
                {
                    SetLoadWith(_db, view);
                    DVEIR_REFERRAL referral;

                    if (id == 0)
                    {
                        referral = new DVEIR_REFERRAL();
                        _db.REFERRALs.Attach(referral);

                        referral.SetAsChangeTrackingRoot(VeteransAffairs.Registries.Business.EntityState.New, true);
                        referral.SetAsInsertOnSubmit();
                    }
                    else
                    {
                        referral = SelectByIDLinqFilter(id).SingleOrDefault();

                        if (referral != null)
                        {
                            referral.SetAsChangeTrackingRoot(true);

                        }

                    }

                    if (referral != null)
                    {
                        int status = referral.EnforceBusinessRules(false, _db);
                    }

                    DVEIRGlobal.CurrentDveivrReferral = referral;
                }
            }

            return DVEIRGlobal.CurrentDveivrReferral;
        }

        /// <summary>
        /// SelectByID with option to get only one encounter in load with
        /// </summary>
        /// <param name="id">referralId</param>
        /// <param name="encounterId">encounterId</param>
        /// <returns></returns>
        public DVEIR_REFERRAL SelectByID(int id, int encounterId, DVEIRReferralView view)
        {
            bool refresh = true;

            if (DVEIRGlobal.CurrentDveivrReferralWithEncounter != null)
            {
                if (DVEIRGlobal.CurrentDveivrReferralWithEncounter.REFERRAL_ID == id
                    && DVEIRGlobal.CurrentDveivrReferralWithEncounter.ER_ENCOUNTERs.Count == 1)
                {
                    if (DVEIRGlobal.CurrentDveivrReferralWithEncounter.ER_ENCOUNTERs.First().ER_ENCOUNTER_ID == encounterId)
                    {
                        refresh = false;
                    }
                }
            }

            if (refresh)
            {
                using (_db = GetDataContext())
                {
                    SetLoadWithOneEncounter(_db, encounterId, view);
                    DVEIR_REFERRAL referral;

                    if (id == 0)
                    {
                        referral = new DVEIR_REFERRAL();
                        _db.REFERRALs.Attach(referral);

                        referral.SetAsChangeTrackingRoot(VeteransAffairs.Registries.Business.EntityState.New, true);
                        referral.SetAsInsertOnSubmit();
                    }
                    else
                    {
                        referral = SelectByIDLinqFilter(id).SingleOrDefault();

                        if (referral != null)
                        {
                            referral.SetAsChangeTrackingRoot(true);

                        }

                    }

                    //get the full visn name for the institution
                    STD_INSTITUTION encounterInstitution
                     = (from inst in _commonManager.GetAllInstitutions() where inst.STATIONNUMBER == referral.ER_ENCOUNTERs[0].STA6A select inst).FirstOrDefault();

                    if (encounterInstitution != null)
                    {
                        referral.ER_ENCOUNTERs[0].FullVisnName = encounterInstitution.FullVISNName;
                    }

                    if (referral != null)
                    {
                        int status = referral.EnforceBusinessRules(false, _db);

                    }

                    DVEIRGlobal.CurrentDveivrReferralWithEncounter = referral;
                }
            }
            return DVEIRGlobal.CurrentDveivrReferralWithEncounter;
        }

        private IQueryable<DVEIR_REFERRAL> SelectByIDLinqFilter(int id)
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();
            
            linqFilter = from t in linqFilter where t.REFERRAL_ID == id select t;

            return linqFilter;
        }

        /// <summary>
        /// Select Method to return IQueryable of Referrals with user access logic in place
        /// </summary>
        /// <returns>IQueryable of DVEIR_REFERRAL</returns>
        public IQueryable<DVEIR_REFERRAL> SelectIQueryable(RegistriesDataAccess _db)
        {
            this._db = _db;

            IQueryable<DVEIR_REFERRAL> entities;
            entities = LinqAll();

            return entities;
        }

        #endregion

        #region Methods for SelectByStatus

        public IEnumerable<DVEIR_REFERRAL> SelectByStatus(int id, string sort, int startRow, int maxRows)
        {
            if ((DVEIRReferralStatus)id == DVEIRReferralStatus.DVEIR_NEW)
            {
                if (String.IsNullOrEmpty(sort))
                    sort = "STD_REFERRALRANK.RANK, PATIENT.LAST_NAME";
                else
                    sort = "STD_REFERRALRANK.RANK, " + sort;
            }
            else if (String.IsNullOrEmpty(sort))
                sort = _defaultSortField;

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                IEnumerable<DVEIR_REFERRAL> entities;
                entities = SelectByStatusLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }
        public int SelectByStatusCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectByStatusLinqFilter(id).Count();
            }
        }
        private IQueryable<DVEIR_REFERRAL> SelectByStatusLinqFilter(int id)
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();

            if( id != 15)
            {
                linqFilter = from t in linqFilter 
                             where t.STD_REFERRALSTS_ID == id
                             select t;
            }
            else
            {
                linqFilter = from t in linqFilter
                             where t.STD_REFERRALSTS_ID == id
                                    && t.UPDATED >=
                                        DateTime.Today.AddDays(-(int)TimePeriodForDisplayingCompletedRecords.Days)
                             select t;
            }

            return linqFilter;
        }

        #endregion
        
        #region Methods for SelectByPatient

        public IEnumerable<DVEIR_REFERRAL> SelectByPatient(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = _defaultSortField;

            }

            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                IEnumerable<DVEIR_REFERRAL> entities;
                entities = SelectByPatientLinqFilter(id).OrderBy(sort).Skip(startRow).Take(maxRows).ToList();

                return entities;
            }
        }


        public int SelectByPatientCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWithForListViews(_db);
                return SelectByPatientLinqFilter(id).Count();
            }
        }

        private IQueryable<DVEIR_REFERRAL> SelectByPatientLinqFilter(int id)
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();

            linqFilter = from t in linqFilter where t.PATIENT_ID == id select t;

            return linqFilter;
        }

        #endregion

        #region Methods for retrieving child entities of DVEIR referral as own list

        public IEnumerable<IDisplayableComment> SelectReviewCommentsByReferralId(int id, string sort, int startRow, int maxRows)
        {
            if (string.IsNullOrEmpty(sort))
            {
                sort = "ID";
            }
           
            using (_db = GetDataContext())
            {
                SetLoadWith(_db, DVEIRReferralView.All);

                return
                    SelectEncounterReviewCommentsByReferralIdFilter(id)
                    .Select(e => new EncounterDisplayableComment
                    {
                        ID = e.ID,
                        Type = "encounter",
                        ReviewCommentEntity = e,
                        EncounterEntity = e.ER_ENCOUNTER
                    }).Cast<IDisplayableComment>().AsEnumerable()
                    .Concat(SelectReferralReviewCommentsByReferralIdFilter(id)
                    .Select(e => new ReferralDisplayableComment
                    {
                        ID = e.ID,
                        Type="referral",
                        ReviewCommentEntity = e,
                        ReferralEntity = e.REFERRAL
                    }).Cast<IDisplayableComment>().AsEnumerable())
                    .OrderBy(x => sort).Skip(startRow).Take(maxRows).ToList();
            }
        }

        private IQueryable<ER_ENCOUNTER_REVIEWCOMMENT> SelectEncounterReviewCommentsByReferralIdFilter(int id)
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();

            linqFilter = from t in linqFilter 
                 where t.REFERRAL_ID == id
                 select t;

            IQueryable<ER_ENCOUNTER_REVIEWCOMMENT> reviewCommentsFlattened =
                    linqFilter.SelectMany(r => r.ER_ENCOUNTERs)
                    .SelectMany(e => e.ER_ENCOUNTER_REVIEWCOMMENTs);

            return reviewCommentsFlattened;
        }

        private IQueryable<ER_REFERRAL_REVIEWCOMMENT> SelectReferralReviewCommentsByReferralIdFilter(int id)
        {
            IQueryable<DVEIR_REFERRAL> linqFilter = LinqAll();

            linqFilter = from t in linqFilter
                         where t.REFERRAL_ID == id
                         select t;

            IQueryable<ER_REFERRAL_REVIEWCOMMENT> reviewCommentsFlattened =
                    linqFilter.SelectMany(r => r.ER_REFERRAL_REVIEWCOMMENTs);

            return reviewCommentsFlattened;
        }

        public int SelectReviewCommentsByReferralIdCount(int id, string sort, int startRow, int maxRows)
        {
            using (_db = GetDataContext())
            {
                SetLoadWith(_db, DVEIRReferralView.All);
                return SelectEncounterReviewCommentsByReferralIdFilter(id).Count() +
                    SelectReferralReviewCommentsByReferralIdFilter(id).Count();
            }
        }

        #endregion

        #region Methods for Update

        /// <summary>
        /// Update Referral. Returns id of referral is save is sucessful, returns 0 if no changes were present, returns -1 if a save error occurred
        /// </summary>
        /// <param name="referral">Instance of DVEIR_REFERRAL to save</param>
        /// <returns>int</returns>
        public int Update(DVEIR_REFERRAL referral)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                referral.SynchroniseWithDataContext(_db); //this line traverses all entities, attaching all of them as appropriate to the data context.

                int status = referral.EnforceBusinessRules(true, _db);
                
                if (referral.BrokenRules.Count == 0)
                {
                    
                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();

                        //call submit changes if changes exist
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }
            return returnStatus;
        }

        public int PerformActivityAndSave(  DVEIR_REFERRAL referral, 
                                            string activityCode
                                         )
        {
            bool okToPerformActivity = false;
            int newReferralStatus = 0;

            #region Switch Activity Code
            switch (activityCode)
            {
                case "DVEIR MARKED READY FOR REVIEW":
                    if (
                            (
                            referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_NEW
                            ||
                            referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_IN_PROCESS
                            ||
                            referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_RETURNED
                            )
                        &&
                            (
                            _user.IsSuperUser
                            ||
                            _user.IsInRole((int)DVEIRRoles.Abstractor)
                            )
                       )
                    {
                        okToPerformActivity = true;
                        newReferralStatus = (int)DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW;
                    }
                    break;
                case "DVEIR SENT BACK TO ABSTRACTOR":
                    if (
                           (
                           referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW
                           ||
                           referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR
                           )
                       &&
                           (
                           _user.IsSuperUser
                           ||
                           _user.IsInRole((int)DVEIRRoles.MedicalProfessional)
                           ||
                           _user.IsInRole((int)DVEIRRoles.Abstractor)
                           )
                      )
                    {
                        okToPerformActivity = true;
                        newReferralStatus = (int)DVEIRReferralStatus.DVEIR_RETURNED;
                        
                        referral.OnUpdate_SetAllEncountersToReadyToSend = false;
                    }
                    break;
                case "DVEIR MARKED READY TO SEND TO DVIER":
                    if (
                            (
                            referral.STD_REFERRALSTS_ID == (int)DVEIRReferralStatus.DVEIR_READY_FOR_REVIEW
                            )
                        &&
                            (
                            _user.IsSuperUser
                            ||
                            _user.IsInRole((int)DVEIRRoles.MedicalProfessional)
                            )
                       )
                    {
                        okToPerformActivity = true;
                        newReferralStatus = (int)DVEIRReferralStatus.DVEIR_READY_TO_SEND_TO_DVEIR;

                        referral.OnUpdate_SetAllEncountersToReadyToSend = true;
                    }
                    break;
            }
            #endregion Switch Activity Code
            
            //Perform Activity
            if (okToPerformActivity)
            {
                return AddActivityAndChangeStatus(activityCode, ref referral, newReferralStatus);
            }
            return 0;
        }

        private int AddActivityAndChangeStatus(string activityCode, ref DVEIR_REFERRAL referral, int newReferralStatus)
        {
            int returnStatus = 0;
            BOSaveSuccessEventArgs eventArgs = new BOSaveSuccessEventArgs();

            using (_db = GetDataContext())
            {
                _db.DeferredLoadingEnabled = false;

                referral.SynchroniseWithDataContext(_db);

                //Add activity
                AddActivity(activityCode, referral);

                //Set Status
                referral.STD_REFERRALST = (from se in _db.STD_REFERRALSTs where se.ID == newReferralStatus select se).FirstOrDefault();

                if (referral.OnUpdate_SetAllEncountersToReadyToSend.HasValue)
                {
                    //set the flag for all encounters currently within the referral's er_encounter entity set
                    foreach (var enc in referral.ER_ENCOUNTERs)
                    {
                        enc.Ready_to_Send_to_DVEIR_FLAG = referral.OnUpdate_SetAllEncountersToReadyToSend;
                    }

                    int referralID = referral.REFERRAL_ID;
                    int[] attachedEncounterIDs = (from e in referral.ER_ENCOUNTERs select e.ER_ENCOUNTER_ID).Distinct().ToArray();
 
                    //Loadwidth will not load all encounters all the time. 
                    //Therefore get all encounters for the referral that are not in the current er_encounter entity set
                    IEnumerable<ER_ENCOUNTER> encountersNotLoaded = (from e in _db.ER_ENCOUNTERs
                                                                     where
                                                                     e.REFERRAL_ID == referralID
                                                                     &&
                                                                     !(attachedEncounterIDs.Contains(e.ER_ENCOUNTER_ID))
                                                                     select e);

                    foreach (var enc in encountersNotLoaded)
                    {
                        enc.Ready_to_Send_to_DVEIR_FLAG = referral.OnUpdate_SetAllEncountersToReadyToSend;
                    }

                }
                
                int status = referral.EnforceBusinessRules(true, _db);

                if (referral.BrokenRules.Count == 0)
                {

                    //Check if any actual changes will occur
                    ChangeSet changeSet = _db.GetChangeSet();

                    if (changeSet.Deletes.Count > 0 || changeSet.Inserts.Count > 0
                    || changeSet.Updates.Count > 0)
                    {
                        _commonManagerDveir.ResetGlobalVariables();
                        //commenting out this section so that concurrent saves on encounters in same referral 
                        //do not cause a linq exception
                        //referral.UPDATEDBY = "ItemUpdate";

                        //call submit changes if changes exist
                        try
                        {
                            _db.SubmitChanges(ConflictMode.ContinueOnConflict);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch (ChangeConflictException)
                        {
                            _db.ChangeConflicts.ResolveAll(RefreshMode.KeepChanges);

                            returnStatus =
                            eventArgs.SavedItemId = referral.REFERRAL_ID;
                            eventArgs.SaveStatusArg = SaveStatus.SaveSuccess;
                            RaiseSaveEvent(this, eventArgs);
                        }
                        catch
                        {
                            returnStatus = -1;
                            eventArgs.SavedItemId = -1;
                            eventArgs.SaveStatusArg = SaveStatus.SaveFail;
                            RaiseSaveEvent(this, eventArgs);
                        }
                    }
                }
            }

            return returnStatus;
        }

        
        private void AddActivity(string activityCode, DVEIR_REFERRAL referral)
        {
            STD_WKFACTIVITYTYPE activityType = (from a in _db.STD_WKFACTIVITYTYPEs where a.CODE == activityCode select a).FirstOrDefault();
            STD_WKFACTIVITYST activityStatus = (from s in _db.STD_WKFACTIVITYSTs where s.CODE == "COMPLETED" select s).FirstOrDefault();

            WKF_CASE_ACTIVITY activity = new WKF_CASE_ACTIVITY();
            activity.STD_WKFACTIVITYTYPE = activityType;
            activity.STD_WKFACTIVITYST = activityStatus;
            activity.REPRESENTATIVE_NAME = "";

            //activity.SetAsInsertOnSubmit();
            _db.WKF_CASE_ACTIVITies.InsertOnSubmit(activity);

            referral.WKF_CASEs[0].WKF_CASE_ACTIVITies.Add(activity);
        }

        #endregion

        #region Methods to change foreign key references
        /// <summary>
        /// Method to return a new selected Referral Status (STD_REFERRALSTs) in order to modify the linq entity.
        /// </summary>
        public STD_REFERRALST getNewReferralStatus(int id)
        {
            STD_REFERRALST newEntity;

            using (_db = GetDataContext())
            {
                newEntity = (from se in _db.STD_REFERRALSTs where se.ID == id select se).FirstOrDefault();
            }

            return newEntity;

        }
        #endregion

        #region EnforceBusinessRules

        public int EnforceBusinessRules(ref DVEIR_REFERRAL referral) 
        {
            using (_db = GetDataContext())
            {
                int status = referral.EnforceBusinessRules(false, _db);
            }

            return 0;
        }


        #endregion
    }

    public interface IDisplayableComment
    {
        int ID { get; }
        string Type { get; }
        string Comment { get; }
        string FriendlyPanelName { get;}
        DateTime Date { get; }
        bool? Resolved { get; }
        bool? Approved { get; }
    }

    public class EncounterDisplayableComment : IDisplayableComment
    {
        public int ID { get; set; }
        public string Type { get; set; }
        public ER_ENCOUNTER_REVIEWCOMMENT ReviewCommentEntity { get; set; }
        public ER_ENCOUNTER EncounterEntity { get; set; }
        public string FriendlyPanelName
        {
            get
            {
                return "Encounter "
                        + this.EncounterEntity.VISIT_DATE.ToShortDateString()
                        + " > "
                        + this.ReviewCommentEntity.LOCATIONPAGE
                        + " > "
                        + this.ReviewCommentEntity.LOCATION_FRIENDLY_NAME;
            }
        }

        public string Comment
        {
            get
            {
                return ReviewCommentEntity.COMMENT_TEXT;
            }
        }

        public DateTime Date
        {
            get
            {
                return ReviewCommentEntity.DATE_ENTERED;
            }
        }

        public bool? Resolved
        {
            get
            {
                return ReviewCommentEntity.RESOLVED_FLAG;
            }
        }

        public bool? Approved
        {
            get
            {
                return ReviewCommentEntity.APPROVED_FLAG;
            }
        }
    }

    public class ReferralDisplayableComment : IDisplayableComment
    {
        public int ID { get; set; }
        public string Type { get; set; }
        public ER_REFERRAL_REVIEWCOMMENT ReviewCommentEntity { get; set; }
        public REFERRAL ReferralEntity { get; set; }
        public string FriendlyPanelName
        {
            get
            {
                return  "Patient > "
                        + this.ReviewCommentEntity.LOCATION_FRIENDLY_NAME;
            }
        }

        public string Comment
        {
            get
            {
                return ReviewCommentEntity.COMMENT_TEXT;
            }
        }

        public DateTime Date
        {
            get
            {
                return ReviewCommentEntity.DATE_ENTERED;
            }
        }

        public bool? Resolved
        {
            get
            {
                return ReviewCommentEntity.RESOLVED_FLAG;
            }
        }

        public bool? Approved
        {
            get
            {
                return ReviewCommentEntity.APPROVED_FLAG;
            }
        }
    }
}
