﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using Telerik.Web.UI.ImageEditor;

namespace DataAccess
{
    /// <summary>
    /// Summary description for Data Access Manager
    /// </summary>
    public class DataAccessManager
    {
        /// <summary>The _current user</summary>
        private readonly string _currentUser;

        /// <summary>Initializes a new instance of the <see cref="DataAccessManager"/> class.</summary>
        public DataAccessManager()
            : this(string.Empty)
        { }

        /// <summary>Initializes a new instance of the <see cref="DataAccessManager"/> class.</summary>
        /// <param name="currentUser">The current user.</param>
        public DataAccessManager(string currentUser)
        {
            this._currentUser = currentUser;
        }

        #region Referral Search Queries

        #endregion

        #region Referral Queries

        internal usp_GetPatientNameByReferralID_Result GetPatientNameByReferralId(int? referralId)
        {
            usp_GetPatientNameByReferralID_Result patientName = null;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    patientName = ctx.usp_GetPatientNameByReferralID(referralId).FirstOrDefault();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return patientName;
        }

        /// <summary>Gets the users by role.</summary>
        /// <param name="userRole">The user role.</param>
        /// <returns></returns>
        internal IEnumerable<usp_GetRegistryUsersByRoleCode_Result> GetUsersByRole(string userRole)
        {
            IEnumerable<usp_GetRegistryUsersByRoleCode_Result> userResults =
                new List<usp_GetRegistryUsersByRoleCode_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    userResults = ctx.usp_GetRegistryUsersByRoleCode(userRole).ToArray();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return userResults;
        }

        /// <summary>Gets the users by role.</summary>
        /// <returns></returns>
        internal IEnumerable<usp_GetAllRegistryUsers_Result> GetAllUsers()
        {
            IEnumerable<usp_GetAllRegistryUsers_Result> userResults =
                new List<usp_GetAllRegistryUsers_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    userResults = ctx.usp_GetAllRegistryUsers().ToArray();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return userResults;
        }

        /// <summary>Gets the encounters by referral identifier.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="statusId">The status identifier.</param>
        /// <returns></returns>
        internal IEnumerable<usp_GetEncountersByReferralId_Result> GetEncountersByReferralId(int referralId, int statusId = 0)
        {
            IEnumerable<usp_GetEncountersByReferralId_Result> encounterResults =
                new List<usp_GetEncountersByReferralId_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    encounterResults = ctx.usp_GetEncountersByReferralId(referralId, statusId).ToArray();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return encounterResults;
        }

        /// <summary>Gets the problem list by referral identifier.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <returns></returns>
        internal IEnumerable<PROBLEM_LIST> GetProblemListByReferralId(int referralId)
        {
            IEnumerable<PROBLEM_LIST> problemList = new List<PROBLEM_LIST>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    problemList = ctx.usp_GetProblemListEntriesByReferralID(referralId).ToArray();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return problemList;
        }

        /// <summary>Gets the patient sid by referral identifier.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <returns></returns>
        internal int GetPatientSIDByReferralId(int referralId)
        {
            int patientSID = 0;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    patientSID = ctx.usp_GetPatientSIDSByReferralID(referralId).FirstOrDefault() ?? 0;
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return patientSID;
        }

        /// <summary>Gets the facility name and sta3n by identifier.</summary>
        /// <param name="institutionId">The institution identifier.</param>
        /// <returns></returns>
        internal usp_GetInstitutionNameAndSta3nByID_Result GetFacilityNameAndSta3nById(int institutionId)
        {
            usp_GetInstitutionNameAndSta3nByID_Result result = new usp_GetInstitutionNameAndSta3nByID_Result();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    result = ctx.usp_GetInstitutionNameAndSta3nByID(institutionId).FirstOrDefault() ??
                             new usp_GetInstitutionNameAndSta3nByID_Result();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return result;
        }

        internal IEnumerable<usp_GetVISNs_Result> GetAllVisns()
        {
            IEnumerable<usp_GetVISNs_Result> result = new usp_GetVISNs_Result[0];

            Action action = () =>
            {
                using (var ctx = new MEVIREntities())
                {
                    result = ctx.usp_GetVISNs().ToList();
                }
            };

            DatabaseCall(action);

            return result;
        }

        internal IEnumerable<usp_Get_Facilities_by_Visn_ID_Result> GetFacilitiesByVisnId(int visnId)
        {
            IEnumerable<usp_Get_Facilities_by_Visn_ID_Result> results =
                new usp_Get_Facilities_by_Visn_ID_Result[0];

            Action dbAction = () =>
            {
                using (var ctx = new MEVIREntities())
                {
                    results = ctx.usp_Get_Facilities_by_Visn_ID(visnId).ToList();
                }
            };

            DatabaseCall(dbAction);

            return results;
        }

        internal IEnumerable<usp_GetReferralsByCriteria_Result> GetReferralsByCriteria(int encounterStatusId, int visnId,
            int facilityId, string patientNameSSN)
        {
            IEnumerable<usp_GetReferralsByCriteria_Result> results = new usp_GetReferralsByCriteria_Result[0];

            Action dbAction = () =>
            {
                using (var ctx = new MEVIREntities())
                {
                    patientNameSSN = string.IsNullOrEmpty(patientNameSSN) ? null : patientNameSSN;
                    results = ctx.usp_GetReferralsByCriteria(encounterStatusId, visnId, facilityId, patientNameSSN).ToList();
                }
            };

            DatabaseCall(dbAction);

            return results;
        }

        #endregion

        #region Encounter Queries

        /// <summary>
        /// GetEncounterById: Returns the ER_ENCOUNTER_MOD record with the passed id (null if not found)
        /// </summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="referralDate">The referral date.</param>
        /// <returns></returns>
        internal ER_ENCOUNTER_MOD GetEncounterById(int referralId, int encounterId, out DateTime referralDate)
        {
            ER_ENCOUNTER_MOD encounter = null;
            referralDate = DateTime.Now;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    encounter =
                        ctx.ER_ENCOUNTER_MOD.FirstOrDefault(
                            e => e.REFERRAL_ID == referralId && e.ER_ENCOUNTER_ID == encounterId);

                    referralDate = ctx.GetReferralDateByEncounterId(encounterId).FirstOrDefault() ?? DateTime.Now;
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return encounter;
        }

        /// <summary>Gets the encounter by identifier.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        internal ER_ENCOUNTER_MOD GetEncounterById(int referralId, int encounterId)
        {
            ER_ENCOUNTER_MOD encounter = null;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    encounter =
                        ctx.ER_ENCOUNTER_MOD.FirstOrDefault(
                            e => e.REFERRAL_ID == referralId && e.ER_ENCOUNTER_ID == encounterId);
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return encounter;
        }

        /// <summary>Gets the encounter by identifier.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        internal ER_ENCOUNTER_MOD GetEncounterById(int encounterId)
        {
            ER_ENCOUNTER_MOD encounter = null;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    encounter = ctx.ER_ENCOUNTER_MOD.FirstOrDefault(e => e.ER_ENCOUNTER_ID == encounterId);
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return encounter;
        }

        /// <summary>Saves the encounter.</summary>
        /// <param name="dbEncounter">The database encounter.</param>
        internal void SaveEncounter(ER_ENCOUNTER_MOD dbEncounter)
        {
            DateTime saveDate = DateTime.Now;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    if (dbEncounter.ER_ENCOUNTER_ID == 0)
                    {
                        dbEncounter.CREATED = saveDate;
                        dbEncounter.CREATEDBY = _currentUser;
                    }

                    dbEncounter.UPDATED = saveDate;
                    dbEncounter.UPDATEDBY = _currentUser;

                    ctx.Entry(dbEncounter).State = dbEncounter.ER_ENCOUNTER_ID == 0 ? EntityState.Added : EntityState.Modified;
                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }
        }

        /// <summary>Encounters the updated.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        internal void EncounterUpdated(int encounterId)
        {
            ER_ENCOUNTER_MOD dbEncounter = GetEncounterById(encounterId);
            SaveEncounter(dbEncounter);
        }

        #endregion

        #region Question / Response

        /// <summary>Gets all lataralities.</summary>
        /// <returns></returns>
        internal IEnumerable<LATERALITY> GetAllLataralities()
        {
            IEnumerable<LATERALITY> lateralityList = new List<LATERALITY>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    lateralityList = ctx.LATERALITIES.ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return lateralityList;
        }

        /// <summary>Gets the questions by category.</summary>
        /// <param name="categoryCode">The category code.</param>
        /// <returns></returns>
        internal IEnumerable<QUESTION> GetQuestionsByCategory(string categoryCode)
        {
            IEnumerable<QUESTION> questionList = new List<QUESTION>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Eager Load question list with choices
                    questionList = ctx.QUESTIONS
                        .Where(q => q.QUESTION_CATEGORY_CODE.Equals(categoryCode))
                        .Include(q => q.QUESTION_CHOICES)
                        .ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return questionList;
        }

        /// <summary>Gets the response list.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <param name="categoryCode">The category code.</param>
        /// <param name="groupingId">The grouping identifier.</param>
        /// <returns></returns>
        internal IEnumerable<RESPONSE> GetResponseList(int referralId, int encounterId, string lateralityCode, string categoryCode, long groupingId)
        {
            IEnumerable<RESPONSE> responseList = new List<RESPONSE>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    responseList = ctx.RESPONSES.Where(
                        r =>
                            r.REFERRAL_ID == referralId &&
                            r.ENCOUNTER_ID == encounterId &&
                            r.LATERALITY_CODE.Equals(lateralityCode) &&
                            r.QUESTION.QUESTION_CATEGORY_CODE.Equals(categoryCode) &&
                            r.GROUPING_ID == groupingId)
                        .Include(r => r.LATERALITY)
                        .Include(r => r.QUESTION)
                        .Include(r => r.QUESTION_CHOICE)
                        .ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return responseList;
        }

        /// <summary>Saves the and replace responses.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="responsesToBeRemoved">The responses to be removed.</param>
        /// <param name="responsesToBeAdded">The responses to be added.</param>
        /// <returns></returns>
        internal long SaveAndReplaceResponses(int encounterId, IEnumerable<RESPONSE> responsesToBeRemoved, IEnumerable<RESPONSE> responsesToBeAdded)
        {
            long groupingId = DateTime.UtcNow.Ticks;
            return SaveAndReplaceResponses(encounterId, groupingId, responsesToBeRemoved, responsesToBeAdded);
        }

        internal long SaveAndReplaceResponses(int encounterId, long groupingId, IEnumerable<RESPONSE> responsesToBeRemoved, IEnumerable<RESPONSE> responsesToBeAdded)
        {
            DateTime? createdDate = DateTime.Now;
            DateTime? updateDate = createdDate;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Set old responses to removed
                    if (responsesToBeRemoved.Any())
                    {
                        createdDate = responsesToBeRemoved.First().CREATED;
                        foreach (var response in responsesToBeRemoved)
                        {
                            ctx.Entry(response).State = EntityState.Deleted;
                        }
                    }

                    // Set new responses to be added
                    foreach (var response in responsesToBeAdded.Where(r => r != null))
                    {
                        response.GROUPING_ID = groupingId;
                        response.CREATED = createdDate;
                        response.CREATED_BY = _currentUser;
                        response.UPDATED = updateDate;
                        response.UPDATED_BY = _currentUser;

                        ctx.RESPONSES.Add(response);
                    }

                    // Save Changes
                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            // Note the encounter was updated
            EncounterUpdated(encounterId);

            return groupingId;
        }

        /// <summary>Removes the responses.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="responsesToBeRemoved">The responses to be removed.</param>
        internal void RemoveResponses(int encounterId, IEnumerable<RESPONSE> responsesToBeRemoved)
        {
            SaveAndReplaceResponses(encounterId, responsesToBeRemoved, new List<RESPONSE>());
        }

        /// <summary>Gets the questoion categories by section.</summary>
        /// <param name="section">The section.</param>
        /// <returns></returns>
        internal IEnumerable<QUESTION_CATEGORY> GetQuestoionCategoriesBySection(string section)
        {
            IEnumerable<QUESTION_CATEGORY> questionCategories = new List<QUESTION_CATEGORY>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Eager Load question list with choices
                    questionCategories = ctx.QUESTION_CATEGORIES.Where(c => c.QUESTION_SECTION_CODE == section).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return questionCategories;
        }

        /// <summary>Gets the responses by encounter.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        internal IEnumerable<RESPONSE> GetResponsesByEncounter(int referralId, int encounterId)
        {
            IEnumerable<RESPONSE> responses = new List<RESPONSE>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Eager Load question list with choices
                    responses = ctx.RESPONSES
                        .Where(r => r.REFERRAL_ID == referralId && r.ENCOUNTER_ID == encounterId)
                        .Include(r => r.QUESTION)
                        .Include(r => r.LATERALITY)
                        .Include(r => r.QUESTION_CHOICE)
                        .ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return responses;
        }

        #endregion

        #region Reviewer Comment

        /// <summary>Gets the review comments by encounter identifier.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        internal IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> GetReviewCommentsByEncounterId(int encounterId)
        {
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = new List<ER_ENCOUNTER_REVIEWCOMMENT>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Eager Load question list with choices
                    reviewComments = ctx.ER_ENCOUNTER_REVIEWCOMMENT
                        .Where(r => r.ER_ENCOUNTER_MOD_ID == encounterId)
                        .ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return reviewComments;
        }

        /// <summary>Gets the review comments by encounter identifier.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="questionCode">The question code.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <param name="groupingId">The grouping identifier.</param>
        /// <returns></returns>
        internal ER_ENCOUNTER_REVIEWCOMMENT GetReviewCommentsByEncounterId(int encounterId, string questionCode, string lateralityCode, long? groupingId)
        {
            ER_ENCOUNTER_REVIEWCOMMENT reviewComment = null;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Eager Load question list with choices
                    reviewComment = ctx.ER_ENCOUNTER_REVIEWCOMMENT
                        .FirstOrDefault(r => r.ER_ENCOUNTER_MOD_ID == encounterId
                                             && r.QUESTION_CODE == questionCode
                                             && r.LATERALITY_CODE == lateralityCode
                                             && (groupingId.HasValue ? r.GROUPING_ID == groupingId.Value : r.GROUPING_ID == null));
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return reviewComment;
        }

        /// <summary>Saves the review comment.</summary>
        /// <param name="dbReviewComment">The database review comment.</param>
        internal void SaveReviewComment(ER_ENCOUNTER_REVIEWCOMMENT dbReviewComment)
        {
            DateTime saveDate = DateTime.Now;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    if (dbReviewComment.ID == 0)
                    {
                        dbReviewComment.CREATED = saveDate;
                        dbReviewComment.CREATEDBY = _currentUser;
                    }

                    dbReviewComment.UPDATED = saveDate;
                    dbReviewComment.UPDATEDBY = _currentUser;

                    ctx.Entry(dbReviewComment).State = dbReviewComment.ID == 0 ? EntityState.Added : EntityState.Modified;

                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }
        }

        #endregion

        #region Encounter Provider

        internal IEnumerable<ENCOUNTER_STAFF> GetEncounterProvidersByVisitId(string visitId)
        {
            IEnumerable<ENCOUNTER_STAFF> providers = new List<ENCOUNTER_STAFF>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    providers = ctx.ENCOUNTER_STAFF.Where(es => es.VisitIdentifier == visitId).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return providers;
        }

        /// <summary>Gets the providers by encounter.</summary>
        /// <param name="visitId">The visit identifier.</param>
        /// <returns></returns>
        internal IEnumerable<usp_GetProvidersByVisitId_Result> GetProvidersByEncounter(string visitId)
        {
            IEnumerable<usp_GetProvidersByVisitId_Result> providers = new List<usp_GetProvidersByVisitId_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    providers = ctx.usp_GetProvidersByVisitId(visitId).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return providers;
        }

        /// <summary>Gets the providers by facility identifier.</summary>
        /// <param name="facilityId">The facility identifier.</param>
        /// <returns></returns>
        internal IEnumerable<PROVIDER> GetProvidersByFacilityId(int facilityId)
        {
            IEnumerable<PROVIDER> providers = new List<PROVIDER>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    providers = ctx.PROVIDERS
                        .Where(s => s.Sta3n == facilityId)
                        .ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return providers;
        }

        /// <summary>Saves the encounter provider.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="visitId">The visit identifier.</param>
        /// <param name="providerId">The provider identifier.</param>
        /// <param name="primarySecondary">The primary or secondary.</param>
        internal void SaveEncounterProvider(int encounterId, string visitId, int providerId, string primarySecondary)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    // Check for the existing record first
                    var encounterProvidersToDelete =
                        ctx.ENCOUNTER_STAFF.Where(s => s.VisitIdentifier == visitId && s.StaffSID == providerId);

                    if (!encounterProvidersToDelete.Any())
                    {
                        // Add record if none exist
                        var encounterStaff = new ENCOUNTER_STAFF()
                        {
                            VisitIdentifier = visitId,
                            StaffSID = providerId,
                            PrimarySecondary = primarySecondary
                        };

                        ctx.ENCOUNTER_STAFF.Add(encounterStaff);
                        ctx.SaveChanges();
                    }
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            EncounterUpdated(encounterId);
        }

        /// <summary>Deletes the encounter provider.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="visitId">The visit identifier.</param>
        /// <param name="providerId">The provider identifier.</param>
        internal void DeleteEncounterProvider(int encounterId, string visitId, int providerId)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    var encounterProvidersToDelete =
                        ctx.ENCOUNTER_STAFF.Where(s => s.VisitIdentifier == visitId && s.StaffSID == providerId);
                    foreach (var encounterProvider in encounterProvidersToDelete)
                    {
                        ctx.ENCOUNTER_STAFF.Remove(encounterProvider);
                    }

                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            EncounterUpdated(encounterId);
        }

        internal void SaveEncounterProviders(int encounterId, IEnumerable<ENCOUNTER_STAFF> encounterProviders)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    foreach (var provider in encounterProviders)
                    {
                        ctx.Entry(provider).State = provider.ID == 0 ? EntityState.Added : EntityState.Modified;
                    }

                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            EncounterUpdated(encounterId);
        }

        #endregion

        #region Encounter Detail

        /// <summary>Gets the encounter details.</summary>
        /// <param name="visitId">The visit identifier.</param>
        /// <param name="detailType">Type of the detail.</param>
        /// <returns></returns>
        internal IEnumerable<ENCOUNTER_DETAILS> GetEncounterDetails(string visitId, string detailType)
        {
            List<ENCOUNTER_DETAILS> encounterDetails = null;

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    encounterDetails =
                        ctx.ENCOUNTER_DETAILS.Where(d => d.VisitIdentifier == visitId && d.Type == detailType)
                            .Include(d => d.LATERALITY).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return encounterDetails;
        }

        /// <summary>Saves the encounter detail.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="dbDetail">The database detail.</param>
        internal void SaveEncounterDetail(int encounterId, ENCOUNTER_DETAILS dbDetail)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    ctx.Entry(dbDetail).State = dbDetail.ID == 0 ? EntityState.Added : EntityState.Modified;
                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            EncounterUpdated(encounterId);
        }

        /// <summary>Removes the encounter detail.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="encounterDetailId">The encounter detail identifier.</param>
        internal void RemoveEncounterDetail(int encounterId, int encounterDetailId)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    var dbDetail = ctx.ENCOUNTER_DETAILS.FirstOrDefault(d => d.ID == encounterDetailId);

                    if (dbDetail != null)
                    {
                        ctx.ENCOUNTER_DETAILS.Remove(dbDetail);
                        ctx.SaveChanges();
                    }
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            EncounterUpdated(encounterId);
        }

        #endregion

        #region Session Logs

        public void AddSessionLog(SESSION_LOG log)
        {
            using (var ctx = new MEVIREntities())
            {
                try
                {
                    log.CREATED = DateTime.Now;
                    log.CREATED_BY = _currentUser;
                    ctx.SESSION_LOG.Add(log);
                    ctx.SaveChanges();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }
        }

        #endregion

        #region Reporting

        public IEnumerable<usp_UserEventLog_Report_Result> GetUserEventLogReport(
            string username, DateTime dateFrom, DateTime dateTo, int? referralId, int? encounterId)
        {
            var results = new List<usp_UserEventLog_Report_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    results = ctx.usp_UserEventLog_Report(username, dateFrom, dateTo, referralId, encounterId).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return results;
        }

        public IEnumerable<usp_DoDInterfaceTransfer_Report_Result> GetDoDInterfaceTransferReport(DateTime dateFrom, DateTime dateTo)
        {
            var results = new List<usp_DoDInterfaceTransfer_Report_Result>();

            using (var ctx = new MEVIREntities())
            {
                try
                {
                    results = ctx.usp_DoDInterfaceTransfer_Report(dateFrom, dateTo).ToList();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(_currentUser, ex.Message);
                }
            }

            return results;
        }

        #endregion

        internal IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> GetEncountersByUserIdAndStatusId(int userId, int statusId)
        {
            IEnumerable<usp_GetEncountersByUserIdAndStatusId_Result> results = 
                new usp_GetEncountersByUserIdAndStatusId_Result[0];

            Action dbAction = () =>
            {
                using (var ctx = new MEVIREntities())
                {
                    results = ctx.usp_GetEncountersByUserIdAndStatusId(userId, statusId).ToArray();
                }
            };

            DatabaseCall(dbAction);

            return results;
        }

        internal IEnumerable<ENCOUNTER_STATUS> GetEncounterStatuses()
        {
            IEnumerable<ENCOUNTER_STATUS> results = new ENCOUNTER_STATUS[0];

            Action dbAction = () =>
            {
                using (var ctx = new MEVIREntities())
                {
                    results = ctx.ENCOUNTER_STATUS.ToList();
                }
            };

            DatabaseCall(dbAction);

            return results;
        }

        private void DatabaseCall(Action dbAction)
        {
            try
            {
                dbAction();
            }
            catch (Exception ex)
            {
                Logging.WriteLogToFile(_currentUser, ex.Message);
            }
        }
    }
}