﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;

namespace Managers
{
    /// <summary>
    /// Summary description for Base View Manager
    /// </summary>
    public class BaseViewManager
    {
        protected UserAccountManager CurrentUser { get; private set; }

        protected DataAccessManager DataAccessManager { get; private set; }

        public BaseViewManager(UserAccountManager currentUser)
            : this(currentUser, new DataAccessManager(currentUser.UserName)) { }

        public BaseViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager)
        {
            this.CurrentUser = currentUser;
            this.DataAccessManager = dataAccessManager;
        }

        protected bool IsEncounterReadOnly(int encounterStatusId)
        {
            EncounterStatusType statusType = GetEncounterStatusType(encounterStatusId);

            bool canEdit = false;

            // Only statuses that allow the possibility of editing
            switch (statusType)
            {
                case EncounterStatusType.New:
                case EncounterStatusType.InProc:
                case EncounterStatusType.SendBack:
                    canEdit =
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR);
                    break;
                case EncounterStatusType.Review:
                    canEdit =
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
                    break;
            }

            return !canEdit;
        }

        protected EncounterStatusType GetEncounterStatusType(int encounterStatusId)
        {
            EncounterStatusType statusType;

            try
            {
                statusType = (EncounterStatusType)encounterStatusId;
            }
            catch
            {
                // Default it to new
                statusType = EncounterStatusType.New;
            }

            return statusType;
        }

        protected string GetIndicatorString(bool indicatorValue)
        {
            return indicatorValue ? "1" : "0";
        }

        protected bool GetIndicatorValue(string indicator)
        {
            return indicator == "1";
        }

        protected List<Laterality> MapToViewModelLateralities(IEnumerable<LATERALITY> lateralities)
        {
            return lateralities.Select(MapToViewModelLaterality).ToList();
        }

        protected Laterality MapToViewModelLaterality(LATERALITY laterality)
        {
            return new Laterality()
            {
                Code = laterality.LATERALITY_CODE,
                DisplayText = laterality.LATERALITY_NAME,
                SortOrder = laterality.LATERALITY_ID
            };
        }

        protected string GetPatientName(int referralId)
        {
            usp_GetPatientNameByReferralID_Result patientName = DataAccessManager.GetPatientNameByReferralId(referralId);

            string patientFullName =
                patientName != null
                    ? string.Format("{0} {1}", patientName.FirstName.Trim(), patientName.LastName.Trim())
                    : string.Empty;

            return patientFullName;
        }

        protected IEnumerable<UserViewModel> GetUsersByRole(string role)
        {

            IEnumerable<usp_GetRegistryUsersByRoleCode_Result> usersByRole = 
                DataAccessManager.GetUsersByRole("DVEIR DATA ABSTRACTOR");

            return usersByRole.Select(u => new UserViewModel()
            {
                Id = u.USER_ID,
                UserName = u.USERNAME,
                FullName = u.FULL_NAME
            });
        } 

        protected Dictionary<int, string> GetEncounterStatuses()
        {
            var statuses = DataAccessManager.GetEncounterStatuses();
            return statuses.ToDictionary(k => k.ID, v => v.DESCRIPTION);
        }
    }
}