﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;
using ViewModels.EncounterDetail;

namespace Managers.Encounter
{
    /// <summary>Summary description for DiagnosisViewManager</summary>
    public class DiagnosisViewManager : BaseViewManager
    {
        /// <summary>The reviewer comment view manager</summary>
        private readonly ReviewerCommentViewManager reviewerCommentViewManager;
        
        public DiagnosisViewManager(UserAccountManager currentUser) : base(currentUser)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
        }

        public DiagnosisViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) : base(currentUser, dataAccessManager)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
        }

        public DiagnosisViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager, ReviewerCommentViewManager reviewerCommentViewManager) : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = reviewerCommentViewManager;
        }

        /// <summary>Gets the diagnosis view model.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public DiagnosisViewModel GetDiagnosisViewModel(int referralId, int encounterId)
        {

            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = DataAccessManager.GetReviewCommentsByEncounterId(encounterId);
            string patientName = GetPatientName(referralId);

            return GetDiagnosisViewModel(patientName, dbEncounter, reviewComments);
        }

        /// <summary>Gets the diagnosis view model.</summary>
        /// <param name="patientName">Name of the patient.</param>
        /// <param name="dbEncounter">The database encounter.</param>
        /// <param name="reviewComments">The review comments.</param>
        /// <returns></returns>
        public DiagnosisViewModel GetDiagnosisViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEncounter,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments)
        {
            var viewModel = new DiagnosisViewModel();

            var dbLateralities = DataAccessManager.GetAllLataralities();

            if (dbEncounter != null)
            {
                var encounterStatusType = GetEncounterStatusType(dbEncounter.EncounterStatusID ?? 0);

                viewModel.ReferralId = dbEncounter.REFERRAL_ID;
                viewModel.EncounterId = dbEncounter.ER_ENCOUNTER_ID;
                viewModel.PatientName = patientName;
                viewModel.IsReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0);
                viewModel.VisitId = dbEncounter.UniqueVHAEncounterIDPrimaryKey;
                viewModel.BlindRehabIndicator = dbEncounter.BLIND_REHAB_INDICATOR;
                viewModel.AllowReviewerComment = reviewerCommentViewManager.ShowReviewerComment(encounterStatusType);
                viewModel.AvailableLateralities = MapToViewModelLateralities(dbLateralities);
                viewModel.InitialLateralityCode = Laterality.LateralityCodes.NotSpecified;

                viewModel.BlindRehabReviewer = viewModel.AllowReviewerComment
                    ? reviewerCommentViewManager.GetReviewerCommentViewModel(viewModel.ReferralId, viewModel.EncounterId, viewModel.PatientName,
                    "ENC_DIAGNOSIS", Laterality.LateralityCodes.NotSpecified, ReviewCommentAnchorType.ENCOUNTER_DIAGNOSIS_TAB, 
                    null, "Diagnosis > Vision Rehabilitation", encounterStatusType, reviewComments)
                    : new ReviewerCommentViewModel();

                viewModel = GetDiagnosisSummaries(viewModel);
            }

            return viewModel;
        }

        /// <summary>Gets the diagnosis summaries.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <returns></returns>
        public DiagnosisViewModel GetDiagnosisSummaries(DiagnosisViewModel viewModel)
        {
            IEnumerable<ENCOUNTER_DETAILS> encounterDetails = DataAccessManager.GetEncounterDetails(viewModel.VisitId, EncounterDetailType.ICD);

            var diagnosisSummaries = encounterDetails.Select(d => new DiagnosisSummary()
            {
                Id = d.ID,
                LateralityCode = d.LATERALITY_CODE,
                Laterality = d.LATERALITY != null ? d.LATERALITY.LATERALITY_NAME : string.Empty,
                Code = d.Value,
                CodeDescription = d.Description,
                CodeType = d.Type
            });

            viewModel.DiagnosisSummaries = diagnosisSummaries.ToList();

            return viewModel;
        }

        /// <summary>Saves the diagnosis detail.</summary>
        /// <param name="viewModel">The view model.</param>
        public void SaveDiagnosisDetail(DiagnosisViewModel viewModel)
        {
            if (viewModel.DiagnosisSummary != null)
            {
                var dbDetail = new ENCOUNTER_DETAILS()
                {
                    VisitIdentifier = viewModel.VisitId,
                    ID = viewModel.DiagnosisSummary.Id,
                    LATERALITY_CODE = viewModel.DiagnosisSummary.LateralityCode,
                    Name = string.Empty,
                    Value = viewModel.DiagnosisSummary.Code,
                    Description = viewModel.DiagnosisSummary.CodeDescription,
                    Type = EncounterDetailType.ICD
                };

                DataAccessManager.SaveEncounterDetail(viewModel.EncounterId, dbDetail);
            }
        }

        /// <summary>Deletes the diagnosis detail.</summary>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="encounterDetailId">The encounter detail identifier.</param>
        public void DeleteDiagnosisDetail(int encounterId, int encounterDetailId)
        {
            DataAccessManager.RemoveEncounterDetail(encounterId, encounterDetailId);
        }

        /// <summary>Saves the diagnosis view.</summary>
        /// <param name="viewModel">The view model.</param>
        public void SaveDiagnosisView(DiagnosisViewModel viewModel)
        {
            var dbEncounter = DataAccessManager.GetEncounterById(viewModel.ReferralId, viewModel.EncounterId);
            dbEncounter.BLIND_REHAB_INDICATOR = viewModel.BlindRehabIndicator;

            DataAccessManager.SaveEncounter(dbEncounter);
        }
    }
}