﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using Factories;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;

namespace Managers.Encounter
{
    public class ExamViewManager : BaseViewManager
    {
        private readonly string questionSectionCode = "Exam";

        private readonly ReviewerCommentViewManager reviewerCommentViewManager;
        
        public ExamViewManager(UserAccountManager currentUser) : base(currentUser)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
        }

        public ExamViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) : base(currentUser, dataAccessManager)
        {
            reviewerCommentViewManager = new ReviewerCommentViewManager(CurrentUser, DataAccessManager);
        }

        public ExamViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager, ReviewerCommentViewManager reviewerCommentViewManager) : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = reviewerCommentViewManager;
        }

        public ExamViewModel GetExamViewModel(int referralId, int encounterId)
        {
            ER_ENCOUNTER_MOD dbEcnounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = DataAccessManager.GetReviewCommentsByEncounterId(encounterId);
            string patientName = GetPatientName(referralId);

            return GetExamViewModel(patientName, dbEcnounter, reviewComments);
        }

        public ExamViewModel GetExamViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEcnounter, 
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments)
        {
            ExamViewModel examViewModel = new ExamViewModel();
            
            if (dbEcnounter != null)
            {
                bool isReadOnly = IsEncounterReadOnly(dbEcnounter.EncounterStatusID ?? 0);
                ViewModels.Encounter.EncounterStatusType encounterStatusType = GetEncounterStatusType(dbEcnounter.EncounterStatusID ?? 0);

                examViewModel = new ExamViewModel()
                {
                    ReferralId = dbEcnounter.REFERRAL_ID,
                    EncounterId = dbEcnounter.ER_ENCOUNTER_ID,
                    PatientName = patientName,
                    IsReadOnly = isReadOnly,
                    EncounterStatusType = encounterStatusType,
                    IsCurrentlyInitialize = false
                };

                List<QUESTION_CATEGORY> questionCategories =
                    DataAccessManager.GetQuestoionCategoriesBySection(questionSectionCode).ToList();
                List<RESPONSE> responses = DataAccessManager.GetResponsesByEncounter(examViewModel.ReferralId, examViewModel.EncounterId).ToList();

                examViewModel.EventCategories = MapToEventCategories(questionCategories);
                examViewModel.EventSummaries = MapToEventSummaries(questionCategories, responses, reviewComments, isReadOnly);
                examViewModel.AllowReviewerComments = reviewerCommentViewManager.ShowReviewerComment(encounterStatusType);
                examViewModel.ReviewerCommentView = new ReviewerCommentViewModel();
            }

            return examViewModel;
        }

        public ExamViewModel RefreshEventSummaries(ExamViewModel examViewModel)
        {
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = 
                DataAccessManager.GetReviewCommentsByEncounterId(examViewModel.EncounterId);
            IEnumerable<QUESTION_CATEGORY> questionCategories =
                DataAccessManager.GetQuestoionCategoriesBySection(questionSectionCode);
            IEnumerable<RESPONSE> responses = DataAccessManager.GetResponsesByEncounter(examViewModel.ReferralId, examViewModel.EncounterId);

            examViewModel.EventSummaries = 
                MapToEventSummaries(questionCategories, responses, reviewComments, examViewModel.IsReadOnly);

            return examViewModel;
        }

        public ExamViewModel RefreshReviewerComment(ExamViewModel viewModel)
        {
            ReviewerCommentViewModel reviewerCommentViewModel = new ReviewerCommentViewModel();

            if (viewModel.HasSelectedCategory && !string.IsNullOrEmpty(viewModel.SelectedLateralityCode) && viewModel.SelectedGroupingId > 0)
            {
                string friendlyLocationName = 
                    string.Format("Exam > {0} ({1})", viewModel.SelectedCategory.Name, viewModel.SelectedLateralityCode);

                var reviewComments = DataAccessManager.GetReviewCommentsByEncounterId(viewModel.EncounterId);

                reviewerCommentViewModel =
                    reviewerCommentViewManager.GetReviewerCommentViewModel(viewModel.ReferralId, viewModel.EncounterId, viewModel.PatientName, 
                        viewModel.SelectedCategoryCode, viewModel.SelectedLateralityCode, ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB, 
                        viewModel.SelectedGroupingId, friendlyLocationName, viewModel.EncounterStatusType, reviewComments);
            }

            viewModel.ReviewerCommentView = reviewerCommentViewModel;

            return viewModel;
        }

        private IEnumerable<ExamViewEventSummary> MapToEventSummaries(
            IEnumerable<QUESTION_CATEGORY> questionCategories, 
            IEnumerable<RESPONSE> responses,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments,
            bool isReadOnly)
        {
            List<ExamViewEventSummary> eventSummaryList = new List<ExamViewEventSummary>();

            var questionCategoriesInResponses = questionCategories.Where(
                c => responses.Any(r => r.QUESTION.QUESTION_CATEGORY_CODE == c.QUESTION_CATEGORY_CODE));
            // Find the distinct grouping ids
            var distinctGroupings = responses.Select(r => r.GROUPING_ID).Distinct();

            foreach (var questionCategory in questionCategoriesInResponses)
            {
                ExamEventFactoryResult eventFactoryResult =
                    ExamEventFactory.GetExamEventFactoryResult(CurrentUser.UserName, questionCategory.QUESTION_CATEGORY_CODE);

                foreach (var groupingId in distinctGroupings)
                {
                    var groupResponses = responses.Where(r => 
                            r.GROUPING_ID == groupingId &&
                            r.QUESTION.QUESTION_CATEGORY_CODE == questionCategory.QUESTION_CATEGORY_CODE);

                    var groupSingleResponse = groupResponses.FirstOrDefault();

                    if (groupSingleResponse != null)
                    {
                        IEnumerable<string> eventDescriptions = eventFactoryResult.EventViewManager.GetEventDescriptions(groupResponses);
                        ER_ENCOUNTER_REVIEWCOMMENT reviewComment = reviewComments.FirstOrDefault(r =>
                            r.LATERALITY_CODE == groupSingleResponse.LATERALITY_CODE &&
                            r.QUESTION_CODE == questionCategory.QUESTION_CATEGORY_CODE &&
                            r.GROUPING_ID == groupSingleResponse.GROUPING_ID);

                        var eventSummary = new ExamViewEventSummary()
                        {
                            Name = questionCategory.CATEGORY_NAME,
                            CategoryCode = questionCategory.QUESTION_CATEGORY_CODE,
                            CategorySortOrder = questionCategory.SORT_ORDER ?? 0,
                            Laterality = groupSingleResponse.LATERALITY.LATERALITY_NAME,
                            LateralityCode = groupSingleResponse.LATERALITY.LATERALITY_CODE,
                            LateralitySortOrder = groupSingleResponse.LATERALITY.LATERALITY_ID,
                            DescriptionList = eventDescriptions,
                            GroupingId = groupSingleResponse.GROUPING_ID,
                            CreatedBy = groupSingleResponse.CREATED_BY,
                            CreatedDate = groupSingleResponse.CREATED ?? DateTime.Now,
                            IsReadOnly = isReadOnly,
                            AllowDelete = true
                        };

                        if (reviewComment != null)
                        {
                            eventSummary.ReviewComment = reviewComment.COMMENT_TEXT;
                            eventSummary.IsResolved = reviewComment.RESOLVED_FLAG ?? false;
                            eventSummary.IsApproved = reviewComment.APPROVED_FLAG ?? false;
                            eventSummary.AllowDelete = false; // Cannot delete an event with a review
                        }

                        eventSummaryList.Add(eventSummary);
                    }
                }
            }

            return eventSummaryList;
        }

        private IEnumerable<ExamEventCategory> MapToEventCategories(IEnumerable<QUESTION_CATEGORY> questionCategories)
        {
            return questionCategories.Select(
                c => new ExamEventCategory()
                {
                    Code = c.QUESTION_CATEGORY_CODE, 
                    Name = c.CATEGORY_NAME,
                    SortOrder = c.SORT_ORDER ?? 0
                }).ToList();
        }
    }
}