﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ViewModels.Encounter;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for ProceduresViewManager
    /// </summary>
    public class HistoryViewManager : BaseViewManager
    {
        private readonly string questionCategoryCode = "History";

        private readonly string questionLateralityCode = Laterality.LateralityCodes.NotSpecified;

        private readonly long groupingId = 0;

        private readonly ReviewerCommentViewManager reviewerCommentViewManager;

        public HistoryViewManager(UserAccountManager currentUser)
            : base(currentUser)
        {
            this.reviewerCommentViewManager = new ReviewerCommentViewManager(currentUser);
        }

        public HistoryViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager)
            : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = new ReviewerCommentViewManager(currentUser, dataAccessManager);
        }

        public HistoryViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager, ReviewerCommentViewManager reviewerCommentViewManager)
            : base(currentUser, dataAccessManager)
        {
            this.reviewerCommentViewManager = reviewerCommentViewManager;
        }

        public HistoryViewModel GetHistoryViewModel(int referralId, int encounterId)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = DataAccessManager.GetReviewCommentsByEncounterId(encounterId);
            string patientName = GetPatientName(referralId);

            return GetHistoryViewModel(patientName, dbEncounter, reviewComments);
        }

        public HistoryViewModel GetHistoryViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEncounter,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewerComments)
        {
            var viewModel = new HistoryViewModel();

            if (dbEncounter != null)
            {

                List<QUESTION> questions =
                    DataAccessManager.GetQuestionsByCategory(questionCategoryCode).ToList();

                List<RESPONSE> responses =
                    DataAccessManager.GetResponseList(
                        dbEncounter.REFERRAL_ID,
                        dbEncounter.ER_ENCOUNTER_ID,
                        questionLateralityCode,
                        questionCategoryCode,
                        groupingId).ToList();

                viewModel.ReferralId = dbEncounter.REFERRAL_ID;
                viewModel.EncounterId = dbEncounter.ER_ENCOUNTER_ID;
                viewModel.PatientName = patientName;
                viewModel.IsReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0);
                viewModel.History = MapToMultiSelectQuestion(questions, responses, HistoryViewModel.QuestionCodes.History);
                viewModel.Injury = MapToMultiSelectQuestion(questions, responses, HistoryViewModel.QuestionCodes.Injury);
                
                // Reviewer Comments
                ViewModels.Encounter.EncounterStatusType encounterStatusType =
                    GetEncounterStatusType(dbEncounter.EncounterStatusID ?? 0);

                viewModel.AllowReviewerComments =
                    reviewerCommentViewManager.ShowReviewerComment(encounterStatusType);

                viewModel.HistoryReviewerComment = viewModel.AllowReviewerComments
                    ? reviewerCommentViewManager.GetReviewerCommentViewModel(
                    dbEncounter.REFERRAL_ID,
                        dbEncounter.ER_ENCOUNTER_ID,
                        patientName,
                        "ENC_HISTORY",
                        questionLateralityCode,
                        ReviewCommentAnchorType.ENCOUNTER_HISTORY_TAB,
                        groupingId,
                        "History Tab > History / Injury",
                        encounterStatusType,
                        reviewerComments)
                    : new ReviewerCommentViewModel();
            }

            return viewModel;
        }

        public void Save(HistoryViewModel viewModel)
        {
            int referralId = viewModel.ReferralId;
            int encounterId = viewModel.EncounterId;

            // Find Responses to remove
            IEnumerable<RESPONSE> responsesToBeRemoved =
                DataAccessManager.GetResponseList(referralId, encounterId, questionLateralityCode, questionCategoryCode, groupingId);

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.History, referralId, encounterId, questionLateralityCode));
            responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.Injury, referralId, encounterId, questionLateralityCode));

            // Save the changes and return new groupingId
            DataAccessManager.SaveAndReplaceResponses(encounterId, groupingId, responsesToBeRemoved, responsesToBeAdded);
        }

        #region Database Entity Parsers

        private QUESTION FindQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            return questionList.FirstOrDefault(q => q.QUESTION_CODE == questionCode);
        }

        private RESPONSE FindResponseByQuestionCode(IEnumerable<RESPONSE> responseList, string questionCode)
        {
            return responseList.FirstOrDefault(r => r.QUESTION_CODE == questionCode);
        }

        private IEnumerable<RESPONSE> FindResponsesByQuestionCode(IEnumerable<RESPONSE> responseList,
            string questionCode)
        {
            return responseList.Where(r => r.QUESTION_CODE == questionCode);
        }

        #endregion

        #region To View Model Mappers

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            var multiSelectQuestion = new MultiSelectQuestion();
            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, multiSelectQuestion);

            multiSelectQuestion.AllChoices.AddRange(MapToChoiceViewModels(question.QUESTION_CHOICES));

            return multiSelectQuestion;
        }

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            var multiSelectQuestion = MapToMultiSelectQuestion(questionList, questionCode);

            IEnumerable<RESPONSE> responses = FindResponsesByQuestionCode(responseList, questionCode).ToList();

            if (responses.Any())
            {
                multiSelectQuestion.SelectedChoices.AddRange(MapToChoiceViewModels(responses));
            }

            return multiSelectQuestion;
        }

        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<QUESTION_CHOICE> questionChoices)
        {
            return questionChoices.Select(MapToChoiceViewModel);
        }

        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<RESPONSE> responses)
        {
            return responses.Select(r => MapToChoiceViewModel(r.QUESTION_CHOICE, r.COMMENT));
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice)
        {
            return MapToChoiceViewModel(questionChoice, string.Empty);
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice, string commentText)
        {
            var choice = new Choice
            {
                Code = questionChoice.QUESTION_CHOICE_CODE,
                DisplayText = questionChoice.DISPLAY_TEXT,
                SortOrder = questionChoice.SORT_ORDER,
                RequiresComment = questionChoice.COMMENT_REQUIRED,
                Comment = commentText
            };

            return choice;
        }

        private void InitializeQuestion(QUESTION dbQuestion, BaseQuestion baseQuestion)
        {
            baseQuestion.Code = dbQuestion.QUESTION_CODE;
            baseQuestion.DisplayText = dbQuestion.QUESTION_TEXT;
        }

        #endregion

        #region To Database Entity Mappers

        protected IEnumerable<RESPONSE> MapMultiSelectQuestionToResponse(MultiSelectQuestion question, int referralId, int encounterId, string lateralityCode)
        {
            List<RESPONSE> responses = new List<RESPONSE>();
            if (question.IsAnswered)
            {
                foreach (Choice choice in question.SelectedChoices)
                {
                    RESPONSE response = MapBaseQuestionToResponse(question, referralId, encounterId, lateralityCode);
                    response.QUESTION_CHOICE_CODE = choice.Code;
                    response.COMMENT = choice.RequiresComment ? choice.Comment : null;

                    responses.Add(response);
                }
            }

            return responses;
        }
        
        private RESPONSE MapBaseQuestionToResponse(BaseQuestion question, int referralId, int encounterId,
            string lateralityCode)
        {
            RESPONSE response = new RESPONSE()
            {
                ENCOUNTER_ID = encounterId,
                REFERRAL_ID = referralId,
                QUESTION_CODE = question.Code,
                LATERALITY_CODE = lateralityCode
            };

            return response;
        }

        #endregion
    }
}