﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using VeteransAffairs.Registries.BusinessManager.Utilities;
using ViewModels.Encounter;
using STD_INSTITUTION = VeteransAffairs.Registries.Business.STD_INSTITUTION;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for Provider View Manager
    /// </summary>
    public class ProviderViewManager : BaseViewManager
    {
        public ProviderViewManager(UserAccountManager currentUser)
            : base(currentUser)
        {
        }

        public ProviderViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager)
            : base(currentUser, dataAccessManager)
        {
        }

        public ProviderViewModel GetProviderViewModel(int referralId, int encounterId, UserAccountManager currentUser)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(referralId, encounterId);
            List<STD_INSTITUTION> facilityList = DropDownLists.GetVISNStationOnlyList();
            string patientName = GetPatientName(referralId);

            return GetProviderViewModel(patientName, dbEncounter, facilityList);
        }

        public ProviderViewModel GetProviderViewModel(
            string patientName,
            ER_ENCOUNTER_MOD dbEncounter,
            IEnumerable<STD_INSTITUTION> facilities)
        {
            string visitId = dbEncounter.UniqueVHAEncounterIDPrimaryKey;
            var providerSummaries = GetProviderSummaries(visitId);

            Dictionary<string, string> facilityList = facilities.ToDictionary(s => s.STATIONNUMBER, s => s.FullVISNName);

            var providerViewModel = new ProviderViewModel()
            {
                ReferralId = dbEncounter.REFERRAL_ID,
                EncounterId = dbEncounter.ER_ENCOUNTER_ID,
                PatientName = patientName,
                EncounterVisitId = dbEncounter.UniqueVHAEncounterIDPrimaryKey,
                CurrentFacility = dbEncounter.STA6A,
                IsReadOnly = IsEncounterReadOnly(dbEncounter.EncounterStatusID ?? 0),
                Facilities = facilityList,
                Providers = new Dictionary<int, string>(),
                ProviderSummaries = providerSummaries.ToList(),
                HasPrimary = providerSummaries.Any(p => p.IsPrimary)
            };

            return providerViewModel;
        }

        private IEnumerable<ProviderSummary> GetProviderSummaries(string visitId)
        {
            IEnumerable<usp_GetProvidersByVisitId_Result> encounterProviders = DataAccessManager.GetProvidersByEncounter(visitId);

            IEnumerable<ProviderSummary> providerSummaries =
                encounterProviders.Select(p => new ProviderSummary()
                {
                    Id = p.StaffSID,
                    IsPrimary = string.Equals(p.PrimarySecondary, "P", StringComparison.InvariantCultureIgnoreCase),
                    Name = p.StaffName,
                    Sta3n = p.Sta3n,
                    InstitutionName = p.InstitutionName,
                    Address1 = p.StreetAddress1,
                    Address2 = p.StreetAddress2,
                    City = p.City,
                    State = p.StateAbbrev,
                    Zip = p.Zip,
                    BusinessPhone = p.OfficePhone,
                    Fax = p.FaxNumber
                });

            return providerSummaries.ToList();
        }

        public ProviderViewModel RefreshProviderSummaries(ProviderViewModel viewModel)
        {
            var providerSummaries = GetProviderSummaries(viewModel.EncounterVisitId);
            viewModel.ProviderSummaries = providerSummaries;
            viewModel.HasPrimary = providerSummaries.Any(p => p.IsPrimary);

            return viewModel;
        }

        public ProviderViewModel GetProvidersByFacility(ProviderViewModel providerViewModel, string selectedFacility)
        {
            int sta6a = 0;

            providerViewModel.Providers = new Dictionary<int, string>();

            if (int.TryParse(selectedFacility, out sta6a))
            {
                var providers = DataAccessManager.GetProvidersByFacilityId(sta6a);
                providerViewModel.Providers = providers
                        .Where(p => providerViewModel.ProviderSummaries.All(ps => ps.Id != p.StaffSID))
                        .ToDictionary(p => p.StaffSID, p => p.StaffName);
            }

            return providerViewModel;
        }

        public void SaveEncounterProvider(ProviderViewModel viewModel, int providerId, bool isPrimary)
        {
            DataAccessManager.SaveEncounterProvider(viewModel.EncounterId, viewModel.EncounterVisitId, providerId, isPrimary ? "P" : "S");
            
            // Update Primary Provider
            if (viewModel.HasPrimary && isPrimary)
            {
                SetEncounterProviderAsPrimary(viewModel, providerId);
            }
        }

        public void DeleteEncounterProvider(ProviderViewModel viewModel, int providerId)
        {
            DataAccessManager.DeleteEncounterProvider(viewModel.EncounterId, viewModel.EncounterVisitId, providerId);
        }

        public void SetEncounterProviderAsPrimary(ProviderViewModel viewModel, int providerId)
        {
            IEnumerable<ENCOUNTER_STAFF> encounterProviders = DataAccessManager.GetEncounterProvidersByVisitId(viewModel.EncounterVisitId);
            foreach (var provider in encounterProviders)
            {
                provider.PrimarySecondary = provider.StaffSID == providerId ? "P" : "S";
            }

            DataAccessManager.SaveEncounterProviders(viewModel.EncounterId, encounterProviders);
        }
    }
}