﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.Business;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels;
using ER_ENCOUNTER_REVIEWCOMMENT = DataAccess.ER_ENCOUNTER_REVIEWCOMMENT;

namespace Managers.Encounter
{
    /// <summary>
    /// Summary description for ReviewerCommentViewManager
    /// </summary>
    public class ReviewerCommentViewManager : BaseViewManager
    {
        public ReviewerCommentViewManager(UserAccountManager currentUser) : base(currentUser)
        {
        }

        public ReviewerCommentViewManager(UserAccountManager currentUser, DataAccessManager dataAccessManager) 
            : base(currentUser, dataAccessManager)
        {
        }

        public ReviewerCommentViewModel GetReviewerCommentViewModel(
            int encounterId,
            string questionCode,
            string lateralityCode,
            int anchorId,
            long? groupingId,
            string friendlyLocationName)
        {
            ER_ENCOUNTER_MOD dbEncounter = DataAccessManager.GetEncounterById(encounterId);
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewerComments =
                DataAccessManager.GetReviewCommentsByEncounterId(encounterId);
            ViewModels.Encounter.EncounterStatusType encounterStatusType =
                GetEncounterStatusType(dbEncounter.EncounterStatusID ?? 0);
            string patientName = GetPatientName(dbEncounter.REFERRAL_ID);


            return GetReviewerCommentViewModel(dbEncounter.REFERRAL_ID, encounterId, patientName, questionCode, lateralityCode, 
                anchorId, groupingId, friendlyLocationName, encounterStatusType, reviewerComments);
        }

        public ReviewerCommentViewModel GetReviewerCommentViewModel(
            int referralId,
            int encounterId,
            string patientName,
            string questionCode,
            string lateralityCode,
            int anchorId,
            long? groupingId,
            string friendlyLocationName,
            ViewModels.Encounter.EncounterStatusType statusType,
            IEnumerable<ER_ENCOUNTER_REVIEWCOMMENT> reviewerComments)
        {
            ReviewerCommentViewModel viewModel = new ReviewerCommentViewModel()
            {
                ReferralId = referralId,
                EncounterId = encounterId,
                PatientName = patientName,
                QuestionCode = questionCode,
                LateralityCode = lateralityCode,
                AnchorId = anchorId,
                GroupingId = groupingId,
                FriendlyLocationName = friendlyLocationName,
                UserName = CurrentUser.UserName,
                ShowOpenButton = false,
                ShowControl = true
            };

            var dbReview = reviewerComments.FirstOrDefault(
                r => r.ER_ENCOUNTER_MOD_ID == encounterId &&
                     r.QUESTION_CODE == questionCode &&
                     r.LATERALITY_CODE == lateralityCode);

            if (dbReview != null)
            {
                viewModel.ShowOpenButton = true;
                viewModel.CommentText = dbReview.COMMENT_TEXT;
                viewModel.IsApproved = dbReview.APPROVED_FLAG ?? false;
                viewModel.IsResolved = dbReview.RESOLVED_FLAG ?? false;
            }

            viewModel.CanEditComment = CanEditComment(statusType);
            viewModel.CanResolve = CanResolve(statusType);
            viewModel.CanApprove = CanApprove(statusType);

            return viewModel;
        }

        public bool ShowReviewerComment(ViewModels.Encounter.EncounterStatusType statusType)
        {
            bool showReviewerComment = false;

            switch (statusType)
            {
                case ViewModels.Encounter.EncounterStatusType.Review:
                    showReviewerComment =
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
                    break;
                case ViewModels.Encounter.EncounterStatusType.SendBack:
                    showReviewerComment =
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR) ||
                        CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
                    break;
                case ViewModels.Encounter.EncounterStatusType.ReadyToSend:
                case ViewModels.Encounter.EncounterStatusType.Completed:
                    showReviewerComment = true;
                    break;
            }

            return showReviewerComment;
        }

        public ReviewerCommentViewModel RefreshCommentViewModel(ReviewerCommentViewModel viewModel)
        {
            ER_ENCOUNTER_REVIEWCOMMENT dbReview =
                DataAccessManager.GetReviewCommentsByEncounterId(
                    viewModel.EncounterId,
                    viewModel.QuestionCode,
                    viewModel.LateralityCode,
                    viewModel.GroupingId);

            if (dbReview != null)
            {
                viewModel.ShowOpenButton = true;
                viewModel.CommentText = dbReview.COMMENT_TEXT;
                viewModel.IsApproved = dbReview.APPROVED_FLAG ?? false;
                viewModel.IsResolved = dbReview.RESOLVED_FLAG ?? false;
            }

            return viewModel;
        }

        public void SaveReviewerComment(ReviewerCommentViewModel viewModel)
        {
            // get ref to old object
            var dbReviewComment =
                DataAccessManager.GetReviewCommentsByEncounterId(
                    viewModel.EncounterId,
                    viewModel.QuestionCode,
                    viewModel.LateralityCode,
                    viewModel.GroupingId);

            // Add if not exisiting
            if (dbReviewComment == null)
            {
                dbReviewComment = new ER_ENCOUNTER_REVIEWCOMMENT()
                {
                    DATE_ENTERED = DateTime.Now,
                    CREATED = DateTime.Now,
                    CREATEDBY = viewModel.UserName,
                    UPDATED = DateTime.Now,
                    UPDATEDBY = viewModel.UserName
                };

                dbReviewComment.ER_ENCOUNTER_MOD_ID = viewModel.EncounterId;
                dbReviewComment.QUESTION_CODE = viewModel.QuestionCode;
                dbReviewComment.LATERALITY_CODE = viewModel.LateralityCode;
                dbReviewComment.GROUPING_ID = viewModel.GroupingId;

                dbReviewComment.ANCHOR_ID = viewModel.AnchorId;
                dbReviewComment.LOCATION_FRIENDLY_NAME = viewModel.FriendlyLocationName;

                dbReviewComment.COMMENT_TEXT = viewModel.CommentText;
            }
            else
            {
                dbReviewComment.UPDATED = DateTime.Now;
                dbReviewComment.UPDATEDBY = viewModel.UserName;
            }

            if (viewModel.CanEditComment)
            {
                dbReviewComment.COMMENT_TEXT = viewModel.CommentText;
            }

            if (viewModel.CanApprove)
            {
                dbReviewComment.APPROVED_FLAG = viewModel.IsApproved;
                dbReviewComment.APPROVED_BY = viewModel.IsApproved ? viewModel.UserName : string.Empty;
                dbReviewComment.DATE_APPROVED = viewModel.IsApproved ? DateTime.Now : (DateTime?)null;
            }

            if (viewModel.CanResolve)
            {
                dbReviewComment.RESOLVED_FLAG = viewModel.IsResolved;
                dbReviewComment.RESOLVED_BY = viewModel.IsResolved ? viewModel.UserName : string.Empty;
                dbReviewComment.DATE_RESOLVED = viewModel.IsResolved ? DateTime.Now : (DateTime?)null;
            }

            DataAccessManager.SaveReviewComment(dbReviewComment);
        }

        private bool CanEditComment(ViewModels.Encounter.EncounterStatusType statusType)
        {
            bool canEditComment = false;

            if (statusType == ViewModels.Encounter.EncounterStatusType.Review)
            {
                canEditComment =
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
            }

            return canEditComment;
        }

        private bool CanResolve(ViewModels.Encounter.EncounterStatusType statusType)
        {
            bool canResolve = false;

            if (statusType == ViewModels.Encounter.EncounterStatusType.SendBack)
            {
                canResolve =
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_DATA_ABSTRACTOR);
            }

            return canResolve;
        }

        private bool CanApprove(ViewModels.Encounter.EncounterStatusType statusType)
        {
            bool canApprove = false;

            if (statusType == ViewModels.Encounter.EncounterStatusType.Review)
            {
                canApprove =
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_ADMINISTRATOR) ||
                    CurrentUser.IsInRole((int)RoleCode.DVEIR_MEDICAL_PROFESSIONAL);
            }

            return canApprove;
        }
    }
}