﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class AngleRecessionEventViewManager : BaseEventViewManager<AngleRecessionEventModel>
    {
        public AngleRecessionEventViewManager(string currentUser) : base(currentUser)
        {
        }

        public override string CategoryCode
        {
            get { return "AngRecesss"; }
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The Angle Recession Event Model</returns>
        public override AngleRecessionEventModel GetDefaultViewModel(AngleRecessionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                AngleRecessionEventModel.QuestionCodes.TestPerformed,
                AngleRecessionEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                AngleRecessionEventModel.QuestionCodes.TestResult,
                AngleRecessionEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, AngleRecessionEventModel.QuestionCodes.EventComment);

            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, AngleRecessionEventModel.QuestionCodes.VisitData);

            viewModel.ClockHours =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.VisitData, AngleRecessionEventModel.QuestionCodes.ClockHourSuffix);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The Angle Recession Event Model</returns>
        public override AngleRecessionEventModel GetLoadedViewModel(AngleRecessionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.VisitData);
            viewModel.ClockHours = MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.VisitData, AngleRecessionEventModel.QuestionCodes.ClockHourSuffix);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(AngleRecessionEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Only procede if is performed
            if (viewModel.TestPerformed.IsAnswered &&
                AngleRecessionEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                // Visit Data
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.VisitData, referralId, encounterId, lateralityCode));

                // Clock Hours
                responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(viewModel.ClockHours, viewModel.VisitData, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, AngleRecessionEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, AngleRecessionEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, AngleRecessionEventModel.QuestionCodes.VisitData));

            return descriptionList;
        }
    }
}