﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels;
using ViewModels.Exam;

namespace Managers.Exam
{
    public abstract class BaseEventViewManager<TResult> : IEventViewManager
        where TResult : BaseEventModel, new()
    {
        /// <summary>Gets the category code.</summary>
        /// <value>The category code.</value>
        public abstract string CategoryCode { get; }

        protected DataAccessManager DataAccessManager { get; private set; }

        protected BaseEventViewManager(string currentUser)
        {
            this.DataAccessManager = new DataAccessManager(currentUser);
        }

        #region Abstract Methods

        public abstract TResult GetDefaultViewModel(TResult viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList);
        public abstract TResult GetLoadedViewModel(TResult viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList);
        public abstract IEnumerable<RESPONSE> GetResponsesToBeAdded(TResult viewModel, int referralId, int encounterId);
        public abstract IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses);

        #endregion

        #region IEventViewManager Implementations

        /// <summary>Gets the default view model.</summary>
        /// <param name="isReadOnly">if set to <c>true</c> [is read only].</param>
        /// <returns></returns>
        BaseEventModel IEventViewManager.GetDefaultViewModel(bool isReadOnly)
        {
            TResult viewModel = new TResult();

            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsReadOnly = isReadOnly;
            viewModel.IsLateralityLocked = false;

            return GetDefaultViewModel(viewModel, categoryQuestionList, lateralityList);
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <param name="groupingId">The grouping identifier.</param>
        /// <param name="isReadOnly">if set to <c>true</c> [is read only].</param>
        /// <returns></returns>
        BaseEventModel IEventViewManager.GetLoadedViewModel(int referralId, int encounterId, string lateralityCode, long groupingId, bool isReadOnly)
        {
            var viewModel = new TResult();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(referralId, encounterId, lateralityCode, CategoryCode, groupingId).ToList();

            viewModel.IsReadOnly = isReadOnly;

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                viewModel.IsLateralityLocked = false;
                viewModel.PreviousLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.GroupingId = groupingId;

                viewModel = GetLoadedViewModel(viewModel, categoryQuestionList, lateralityList, responseList);
            }
            else
            {
                viewModel = GetDefaultViewModel(viewModel, categoryQuestionList, lateralityList);
            }

            return viewModel;
        }

        /// <summary>Saves the specified event model.</summary>
        /// <param name="eventModel">The event model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        BaseEventModel IEventViewManager.Save(BaseEventModel eventModel, int referralId, int encounterId)
        {
            TResult result = eventModel as TResult;

            if (result != null)
            {
                // Create Responses
                string previousLateralityCode = result.PreviousLaterality != null ? result.PreviousLaterality.Code : string.Empty;
                string currentLateralityCode = result.SelectedLaterality != null ? result.SelectedLaterality.Code : string.Empty;
                string categoryCode = CategoryCode;
                long groupingId = result.GroupingId;

                // Find Responses to remove
                IEnumerable<RESPONSE> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(referralId, encounterId, previousLateralityCode, categoryCode, groupingId);

                // Create Responses To Added
                IEnumerable<RESPONSE> responsesToBeAdded = GetResponsesToBeAdded(result, referralId, encounterId);

                // Save the changes and return new groupingId
                result.GroupingId = DataAccessManager.SaveAndReplaceResponses(encounterId, responsesToBeRemoved, responsesToBeAdded);

                // Update ReviewerCmt
                if (previousLateralityCode != currentLateralityCode || groupingId != result.GroupingId)
                {
                    var reviewerCmt =
                        DataAccessManager.GetReviewCommentsByEncounterId(encounterId, categoryCode, previousLateralityCode, groupingId);
                    
                    if (reviewerCmt != null)
                    {
                        reviewerCmt.LATERALITY_CODE = currentLateralityCode;
                        reviewerCmt.GROUPING_ID = result.GroupingId;
                        reviewerCmt.LOCATION_FRIENDLY_NAME =
                            reviewerCmt.LOCATION_FRIENDLY_NAME.Replace(
                                "(" + previousLateralityCode + ")",
                                "(" + currentLateralityCode + ")");

                        DataAccessManager.SaveReviewComment(reviewerCmt);
                    }
                }
            }

            return result;
        }

        /// <summary>Deletes the specified referral identifier.</summary>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <param name="categoryCode">The category code.</param>
        /// <param name="groupingId">The grouping identifier.</param>
        public void Delete(int referralId, int encounterId, string lateralityCode, string categoryCode, long groupingId)
        {
            // Find Responses to remove
            IEnumerable<RESPONSE> responsesToBeRemoved =
                DataAccessManager.GetResponseList(referralId, encounterId, lateralityCode, categoryCode, groupingId);

            DataAccessManager.RemoveResponses(encounterId, responsesToBeRemoved);
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        IEnumerable<string> IEventViewManager.GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            return GetEventDescriptions(responses).Where(d => !string.IsNullOrEmpty(d));
        }

        #endregion

        #region Database Entity Parsers

        private QUESTION FindQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            return questionList.FirstOrDefault(q => q.QUESTION_CODE == questionCode);
        }

        private RESPONSE FindResponseByQuestionCode(IEnumerable<RESPONSE> responseList, string questionCode)
        {
            return responseList.FirstOrDefault(r => r.QUESTION_CODE == questionCode);
        }

        private IEnumerable<RESPONSE> FindResponsesByQuestionCode(IEnumerable<RESPONSE> responseList,
            string questionCode)
        {
            return responseList.Where(r => r.QUESTION_CODE == questionCode);
        }

        #endregion

        #region To View Model Mappers

        protected List<Laterality> MapToViewModelLateralities(IEnumerable<LATERALITY> lateralities)
        {
            return lateralities.Select(MapToViewModelLaterality).ToList();
        }

        protected Laterality MapToViewModelLaterality(LATERALITY laterality)
        {
            return new Laterality()
            {
                Code = laterality.LATERALITY_CODE,
                DisplayText = laterality.LATERALITY_NAME,
                SortOrder = laterality.LATERALITY_ID
            };
        }

        protected Laterality MapToViewModelLaterality(IEnumerable<LATERALITY> lateralityList, string specifiedCode)
        {
            var laterality = lateralityList.First(l => l.LATERALITY_CODE == specifiedCode);
            return MapToViewModelLaterality(laterality);
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            var singleSelectQuestion = new SingleSelectQuestion();
            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, singleSelectQuestion);

            singleSelectQuestion.AllChoices.AddRange(MapToChoiceViewModels(question.QUESTION_CHOICES));

            return singleSelectQuestion;
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode, string selectedChoiceCode)
        {
            var singleSelectQuestion = MapToSingleSelectQuestion(questionList, questionCode);

            singleSelectQuestion.SelectedChoice =
                singleSelectQuestion.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

            return singleSelectQuestion;
        }

        protected SingleSelectQuestion MapToSingleSelectQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            SingleSelectQuestion singleSelectQuestion = MapToSingleSelectQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                singleSelectQuestion.SelectedChoice = MapToChoiceViewModel(response.QUESTION_CHOICE, response.COMMENT);
            }

            return singleSelectQuestion;
        }

        protected TextQuestion MapToTextQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            TextQuestion textQuestion = new TextQuestion();

            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, textQuestion);

            return textQuestion;
        }

        protected TextQuestion MapToTextQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            TextQuestion textQuestion = MapToTextQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                textQuestion.ResponseText = response.RESPONSE_TEXT;
            }

            return textQuestion;
        }

        protected Dictionary<string, TextQuestion> MapToTextQuestionsMultiSelectSub(
            IEnumerable<QUESTION> questionList,
            MultiSelectQuestion multiSelectQuestion,
            string questionSuffix)
        {
            Dictionary<string, TextQuestion> textQuestions = new Dictionary<string, TextQuestion>();

            foreach (Choice choice in multiSelectQuestion.AllChoices)
            {
                string questionCode = choice.Code + questionSuffix;
                TextQuestion textQuestion = MapToTextQuestion(questionList, questionCode);

                textQuestions.Add(choice.Code, textQuestion);
            }

            return textQuestions;
        }

        protected Dictionary<string, TextQuestion> MapToTextQuestionsMultiSelectSub(
            IEnumerable<QUESTION> questionList,
            IEnumerable<RESPONSE> responseList,
            MultiSelectQuestion multiSelectQuestion,
            string questionSuffix)
        {
            Dictionary<string, TextQuestion> textQuestions = new Dictionary<string, TextQuestion>();

            foreach (Choice choice in multiSelectQuestion.AllChoices)
            {
                string questionCode = choice.Code + questionSuffix;
                TextQuestion textQuestion = MapToTextQuestion(questionList, responseList, questionCode);

                textQuestions.Add(choice.Code, textQuestion);
            }

            return textQuestions;
        }

        protected YesNoQuestion MapToYesNoQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            YesNoQuestion yesNoQuestion = new YesNoQuestion();

            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, yesNoQuestion);

            return yesNoQuestion;
        }

        protected YesNoQuestion MapToYesNoQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            YesNoQuestion yesNoQuestion = MapToYesNoQuestion(questionList, questionCode);

            RESPONSE response = FindResponseByQuestionCode(responseList, questionCode);

            if (response != null)
            {
                yesNoQuestion.ResponseText = response.RESPONSE_TEXT;
            }

            return yesNoQuestion;
        }

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, string questionCode)
        {
            var multiSelectQuestion = new MultiSelectQuestion();
            QUESTION question = FindQuestion(questionList, questionCode);

            InitializeQuestion(question, multiSelectQuestion);

            multiSelectQuestion.AllChoices.AddRange(MapToChoiceViewModels(question.QUESTION_CHOICES));

            return multiSelectQuestion;
        }

        protected MultiSelectQuestion MapToMultiSelectQuestion(IEnumerable<QUESTION> questionList, IEnumerable<RESPONSE> responseList, string questionCode)
        {
            var multiSelectQuestion = MapToMultiSelectQuestion(questionList, questionCode);

            IEnumerable<RESPONSE> responses = FindResponsesByQuestionCode(responseList, questionCode).ToList();

            if (responses.Any())
            {
                multiSelectQuestion.SelectedChoices.AddRange(MapToChoiceViewModels(responses));
            }

            return multiSelectQuestion;
        }

        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<QUESTION_CHOICE> questionChoices)
        {
            return questionChoices.Select(MapToChoiceViewModel);
        }

        protected IEnumerable<Choice> MapToChoiceViewModels(IEnumerable<RESPONSE> responses)
        {
            return responses.Select(r => MapToChoiceViewModel(r.QUESTION_CHOICE, r.COMMENT));
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice)
        {
            return MapToChoiceViewModel(questionChoice, string.Empty);
        }

        protected Choice MapToChoiceViewModel(QUESTION_CHOICE questionChoice, string commentText)
        {
            var choice = new Choice
            {
                Code = questionChoice.QUESTION_CHOICE_CODE,
                DisplayText = questionChoice.DISPLAY_TEXT,
                SortOrder = questionChoice.SORT_ORDER,
                RequiresComment = questionChoice.COMMENT_REQUIRED,
                Comment = commentText
            };

            return choice;
        }

        private void InitializeQuestion(QUESTION dbQuestion, BaseQuestion baseQuestion)
        {
            baseQuestion.Code = dbQuestion.QUESTION_CODE;
            baseQuestion.DisplayText = dbQuestion.QUESTION_TEXT;
        }

        #endregion

        #region To Database Entity Mappers

        protected RESPONSE MapSingleSelectQuestionToResponse(SingleSelectQuestion question, int referralId,
            int encounterId, string lateralityCode)
        {
            RESPONSE response = null;
            if (question.IsAnswered)
            {
                response = MapBaseQuestionToResponse(question, referralId, encounterId, lateralityCode);
                response.QUESTION_CHOICE_CODE = question.SelectedChoice.Code;
                response.COMMENT = question.SelectedChoice.RequiresComment ? question.SelectedChoice.Comment : null;
            }

            return response;
        }

        protected RESPONSE MapTextQuestionToResponse(TextQuestion question, int referralId, int encounterId, string lateralityCode)
        {
            RESPONSE response = null;
            if (question.IsAnswered)
            {
                response = MapBaseQuestionToResponse(question, referralId, encounterId, lateralityCode);
                response.RESPONSE_TEXT = question.ResponseText;
            }

            return response;
        }

        protected RESPONSE MapYesNoQuestionToResponse(YesNoQuestion question, int referralId, int encounterId, string lateralityCode)
        {
            return MapTextQuestionToResponse(question, referralId, encounterId, lateralityCode);
        }

        protected IEnumerable<RESPONSE> MapMultiSelectQuestionToResponse(MultiSelectQuestion question, int referralId, int encounterId, string lateralityCode)
        {
            List<RESPONSE> responses = new List<RESPONSE>();
            if (question.IsAnswered)
            {
                foreach (Choice choice in question.SelectedChoices)
                {
                    RESPONSE response = MapBaseQuestionToResponse(question, referralId, encounterId, lateralityCode);
                    response.QUESTION_CHOICE_CODE = choice.Code;
                    response.COMMENT = choice.RequiresComment ? choice.Comment : null;

                    responses.Add(response);
                }
            }

            return responses;
        }

        protected IEnumerable<RESPONSE> MapTextQuestionMultiSelectSubsToResponses(
            Dictionary<string, TextQuestion> textQuestions,
            MultiSelectQuestion multiSelectQuestion,
            int referralId,
            int encounterId,
            string lateralityCode)
        {
            return multiSelectQuestion.SelectedChoices.Select(
                c =>
                {
                    var textQuestion = textQuestions[c.Code];
                    return MapTextQuestionToResponse(textQuestion, referralId, encounterId, lateralityCode);
                }).ToList();
        }

        private RESPONSE MapBaseQuestionToResponse(BaseQuestion question, int referralId, int encounterId,
            string lateralityCode)
        {
            RESPONSE response = new RESPONSE()
            {
                ENCOUNTER_ID = encounterId,
                REFERRAL_ID = referralId,
                QUESTION_CODE = question.Code,
                LATERALITY_CODE = lateralityCode
            };

            return response;
        }

        #endregion

        #region Utilities

        protected string GetTextQuestionDescriptionLine(IEnumerable<RESPONSE> responses, string questionCode)
        {
            string description = string.Empty;
            var response = responses.FirstOrDefault(r => r.QUESTION_CODE == questionCode);

            if (response != null)
            {
                description =
                    response.QUESTION.QUESTION_TEXT
                    + ": "
                    + response.RESPONSE_TEXT;
            }

            return description;
        }

        protected string GetSingleSelectQuestionDescriptionLine(IEnumerable<RESPONSE> responses, string questionCode)
        {
            string description = string.Empty;
            var response = responses.FirstOrDefault(r => r.QUESTION_CODE == questionCode);

            if (response != null)
            {
                description =
                    response.QUESTION.QUESTION_TEXT
                    + ": "
                    + response.QUESTION_CHOICE.DISPLAY_TEXT;
            }

            return description;
        }

        protected string GetMultiSelectQuestionDescriptionLine(IEnumerable<RESPONSE> responses, string questionCode)
        {
            string description = string.Empty;
            var selectedResponses = responses.Where(r => r.QUESTION_CODE == questionCode);

            int count = 0;
            if (selectedResponses.Any())
            {
                description = selectedResponses.First().QUESTION.QUESTION_TEXT + ": ";

                foreach (var selectedResponse in selectedResponses)
                {
                    description +=
                        (count > 0 ? ", " : string.Empty)
                        + selectedResponse.QUESTION_CHOICE.DISPLAY_TEXT;

                    count++;
                    if (count == 3)
                    {
                        break;
                    }
                }
            }

            return description;
        }

        #endregion
    }
}