﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>Summary description for ExamViewManager</summary>
    public class BrowEventViewManager : BaseEventViewManager<BrowEventModel>
    {
        public BrowEventViewManager(string currentUser) : base(currentUser)
        {
        }

        public override string CategoryCode
        {
            get { return "Brow"; }
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The Brow Event Model</returns>
        public override BrowEventModel GetDefaultViewModel(BrowEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                BrowEventModel.QuestionCodes.TestPerformed,
                BrowEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                BrowEventModel.QuestionCodes.TestResult,
                BrowEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, BrowEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, BrowEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The Brow Event Model</returns>
        public override BrowEventModel GetLoadedViewModel(BrowEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, BrowEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, BrowEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, BrowEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, responseList, BrowEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(BrowEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                BrowEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                // Visit Data
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.VisitData, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, BrowEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, BrowEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, BrowEventModel.QuestionCodes.VisitData));

            return descriptionList;
        }
    }
}