﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ColorVisionEventViewManager
    /// </summary>
    public class ColorVisionEventViewManager : BaseEventViewManager<ColorVisionEventModel>
    {
        public ColorVisionEventViewManager(string currentUser) : base(currentUser)
        {
        }

        public override string CategoryCode
        {
            get { return "ColorVis"; }
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The Color Vision Event Model</returns>
        public override ColorVisionEventModel GetDefaultViewModel(ColorVisionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            // Defaults to both eyes
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                ColorVisionEventModel.QuestionCodes.TestPerformed,
                ColorVisionEventModel.ChoiceCodes.TestPerformed.PIP);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.EventComment);

            viewModel.NumberCorrect = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.NumberCorrect);

            viewModel.TotalNumber = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.TotalNumber);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The Color Vision Event Model</returns>
        public override ColorVisionEventModel GetLoadedViewModel(ColorVisionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.TestPerformed);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.EventComment);
            viewModel.NumberCorrect = MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.NumberCorrect);
            viewModel.TotalNumber = MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.TotalNumber);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(ColorVisionEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                ColorVisionEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Number Correct
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.NumberCorrect, referralId, encounterId, lateralityCode));

                // Total Number
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.TotalNumber, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, ColorVisionEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetColorVision(responses));

            return descriptionList;
        }

        /// <summary>Gets the color vision.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        private string GetColorVision(IEnumerable<RESPONSE> responses)
        {
            string description = string.Empty;
            RESPONSE numberColors = responses.FirstOrDefault(r => r.QUESTION_CODE == ColorVisionEventModel.QuestionCodes.NumberCorrect);
            RESPONSE totalColors = responses.FirstOrDefault(r => r.QUESTION_CODE == ColorVisionEventModel.QuestionCodes.TotalNumber);

            if (numberColors != null && totalColors != null)
            {
                description =
                    "Color Vision: "
                    + numberColors.RESPONSE_TEXT
                    + " / "
                    + totalColors.RESPONSE_TEXT;
            }

            return description;
        }
    }
}