﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class ConfrontationalVisualFieldEventViewManager : BaseEventViewManager<ConfrontationalVisualFieldEventModel>
    {
        public ConfrontationalVisualFieldEventViewManager(string currentUser) : base(currentUser)
        {
        }

        public override string CategoryCode
        {
            get { return "ConfrontVisF"; }
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The Confrontational Visual Field Event Model</returns>
        public override ConfrontationalVisualFieldEventModel GetDefaultViewModel(ConfrontationalVisualFieldEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                ConfrontationalVisualFieldEventModel.QuestionCodes.TestPerformed,
                ConfrontationalVisualFieldEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                ConfrontationalVisualFieldEventModel.QuestionCodes.TestResult,
                ConfrontationalVisualFieldEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, ConfrontationalVisualFieldEventModel.QuestionCodes.EventComment);
            viewModel.Quadrant = MapToMultiSelectQuestion(categoryQuestionList, ConfrontationalVisualFieldEventModel.QuestionCodes.Quadrant);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The Confrontational Visual Field Event Model</returns>
        public override ConfrontationalVisualFieldEventModel GetLoadedViewModel(ConfrontationalVisualFieldEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, ConfrontationalVisualFieldEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, ConfrontationalVisualFieldEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, ConfrontationalVisualFieldEventModel.QuestionCodes.EventComment);
            viewModel.Quadrant = MapToMultiSelectQuestion(categoryQuestionList, responseList, ConfrontationalVisualFieldEventModel.QuestionCodes.Quadrant);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(ConfrontationalVisualFieldEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                ConfrontationalVisualFieldEventModel.ShowTestResultValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                if (viewModel.TestResult.IsAnswered &&
                    ConfrontationalVisualFieldEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestResult.SelectedChoice.Code))
                {
                    // Quadrant
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.Quadrant, referralId, encounterId, lateralityCode));
                }
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, ConfrontationalVisualFieldEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, ConfrontationalVisualFieldEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, ConfrontationalVisualFieldEventModel.QuestionCodes.Quadrant));

            return descriptionList;
        }
    }
}