﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class DepthPerceptionEventViewManager : BaseEventViewManager<DepthPerceptionEventModel>
    {
        public override string CategoryCode
        {
            get { return "DepthPerc"; }
        }

        public DepthPerceptionEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The Depth Perception Event Model</returns>
        public override DepthPerceptionEventModel GetDefaultViewModel(DepthPerceptionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                DepthPerceptionEventModel.QuestionCodes.TestPerformed,
                DepthPerceptionEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                DepthPerceptionEventModel.QuestionCodes.TestResult,
                DepthPerceptionEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.EventComment);
            viewModel.RandotFly = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.RandotFly);
            viewModel.VerhoeffSteropter = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VerhoeffSteropter);
            viewModel.RandotStereoTest = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.RandotStereoTest);
            viewModel.TitmusStereoTest = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.TitmusStereoTest);
            viewModel.VectographicProjectorSlides = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VectographicProjectorSlides);
            viewModel.VisionTesterDepth = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VisionTesterDepth);
            viewModel.BorishVectographicNearpoint = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.BorishVectographicNearpoint);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The Depth Perception Event Model</returns>
        public override DepthPerceptionEventModel GetLoadedViewModel(DepthPerceptionEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TestResult);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.EventComment);
            viewModel.RandotFly = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.RandotFly);
            viewModel.VerhoeffSteropter = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VerhoeffSteropter);
            viewModel.RandotStereoTest = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.RandotStereoTest);
            viewModel.TitmusStereoTest = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TitmusStereoTest);
            viewModel.VectographicProjectorSlides = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VectographicProjectorSlides);
            viewModel.VisionTesterDepth = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VisionTesterDepth);
            viewModel.BorishVectographicNearpoint = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.BorishVectographicNearpoint);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(DepthPerceptionEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                DepthPerceptionEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Randot Stereo Test
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.RandotStereoTest, referralId,
                    encounterId, lateralityCode));

                // Randot Fly
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.RandotFly, referralId, encounterId,
                    lateralityCode));

                // Titmus Stereo Test 
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TitmusStereoTest, referralId,
                    encounterId, lateralityCode));

                // Vectographic Projector Slides
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.VectographicProjectorSlides,
                    referralId, encounterId, lateralityCode));

                // Vision Tester Depth Perception Slide
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.VisionTesterDepth, referralId,
                    encounterId, lateralityCode));

                // Borish Vectographic Nearpoint Card
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.BorishVectographicNearpoint,
                    referralId, encounterId, lateralityCode));

                // Verhoeff Steropter
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.VerhoeffSteropter, referralId, encounterId,
                    lateralityCode));
            }

            return responsesToBeAdded;
        }

        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, DepthPerceptionEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, DepthPerceptionEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetAngleOfStereopsis(responses, DepthPerceptionEventModel.QuestionCodes.RandotStereoTest));
            descriptionList.Add(GetAngleOfStereopsis(responses, DepthPerceptionEventModel.QuestionCodes.TitmusStereoTest));
            descriptionList.Add(GetAngleOfStereopsis(responses, DepthPerceptionEventModel.QuestionCodes.VectographicProjectorSlides));
            descriptionList.Add(GetAngleOfStereopsis(responses, DepthPerceptionEventModel.QuestionCodes.VisionTesterDepth));
            descriptionList.Add(GetAngleOfStereopsis(responses, DepthPerceptionEventModel.QuestionCodes.BorishVectographicNearpoint));

            return descriptionList;
        }

        private string GetAngleOfStereopsis(IEnumerable<RESPONSE> responses, string questionCode)
        {

            string description = string.Empty;
            var response = responses.FirstOrDefault(r => r.QUESTION_CODE == questionCode);

            if (response != null)
            {
                description =
                    response.QUESTION.QUESTION_TEXT
                    + ": "
                    + response.QUESTION_CHOICE.CHOICE_NAME
                    + " seconds of arc";
            }

            return description;
        }
    }
}