﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Globe Event View Manager
    /// </summary>
    public class GlobeEventViewManager : BaseEventViewManager<GlobeEventModel>
    {
        public override string CategoryCode
        {
            get { return "Globe"; }
        }

        public GlobeEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The globe view model</returns>
        public override GlobeEventModel GetDefaultViewModel(GlobeEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                GlobeEventModel.QuestionCodes.TestPerformed,
                GlobeEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                GlobeEventModel.QuestionCodes.TestResult,
                GlobeEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, GlobeEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, GlobeEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The globe view model</returns>
        public override GlobeEventModel GetLoadedViewModel(GlobeEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));

            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, GlobeEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, GlobeEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, GlobeEventModel.QuestionCodes.EventComment);
            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, responseList, GlobeEventModel.QuestionCodes.VisitData);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(GlobeEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                GlobeEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                // Visit Data
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.VisitData, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, GlobeEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, GlobeEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, GlobeEventModel.QuestionCodes.VisitData));

            return descriptionList;
        }
    }
}