﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Intraocular Pressure Event View Manager
    /// </summary>
    public class IntraocularPressureEventViewManager : BaseEventViewManager<IntraocularPressureEventModel>
    {
        public override string CategoryCode
        {
            get { return "IntraocularPres"; }
        }

        public IntraocularPressureEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns>The intraocular pressure event model</returns>
        public override IntraocularPressureEventModel GetDefaultViewModel(IntraocularPressureEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                IntraocularPressureEventModel.QuestionCodes.TestPerformed,
                IntraocularPressureEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                IntraocularPressureEventModel.QuestionCodes.TestResult,
                IntraocularPressureEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.EventComment);
            viewModel.Method = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Method);
            viewModel.MethodComment = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.MethodComment);
            viewModel.Number = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Number);
            viewModel.Time = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Time);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns>The intraocular pressure event model</returns>
        public override IntraocularPressureEventModel GetLoadedViewModel(IntraocularPressureEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.EventComment);

            viewModel.Method = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Method);
            viewModel.MethodComment = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.MethodComment);
            viewModel.Number = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Number);
            viewModel.Time = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Time);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(IntraocularPressureEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                IntraocularPressureEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Method
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Method, referralId, encounterId,
                    lateralityCode));

                // Method Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.MethodComment, referralId, encounterId,
                    lateralityCode));

                // Number
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Number, referralId, encounterId,
                    lateralityCode));

                // Time
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Time, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, IntraocularPressureEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, IntraocularPressureEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, IntraocularPressureEventModel.QuestionCodes.Method));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, IntraocularPressureEventModel.QuestionCodes.Number));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, IntraocularPressureEventModel.QuestionCodes.Time));

            return descriptionList;
        }
    }
}