﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class KineticVisualFieldsEventViewManager : BaseEventViewManager<KineticVisualFieldsEventModel>
    {
        public override string CategoryCode
        {
            get { return "KinecticVisF"; }
        }

        public KineticVisualFieldsEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override KineticVisualFieldsEventModel GetDefaultViewModel(KineticVisualFieldsEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality =
                MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                KineticVisualFieldsEventModel.QuestionCodes.TestPerformed,
                KineticVisualFieldsEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                KineticVisualFieldsEventModel.QuestionCodes.TestResult,
                KineticVisualFieldsEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, KineticVisualFieldsEventModel.QuestionCodes.EventComment);
            viewModel.Methodology = MapToMultiSelectQuestion(categoryQuestionList, KineticVisualFieldsEventModel.QuestionCodes.Methodology);

            viewModel.Results =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.Methodology, KineticVisualFieldsEventModel.QuestionCodes.ResultSuffix);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override KineticVisualFieldsEventModel GetLoadedViewModel(KineticVisualFieldsEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed =
                MapToSingleSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.TestPerformed);

            viewModel.TestResult =
                MapToSingleSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.TestResult);

            viewModel.EventComment =
                MapToTextQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.EventComment);

            viewModel.Methodology =
                MapToMultiSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.Methodology);

            viewModel.Results =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.Methodology, KineticVisualFieldsEventModel.QuestionCodes.ResultSuffix);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(KineticVisualFieldsEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                KineticVisualFieldsEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Methodology
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.Methodology, referralId,
                    encounterId, lateralityCode));

                // Results
                responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(viewModel.Results,
                    viewModel.Methodology, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, KineticVisualFieldsEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, KineticVisualFieldsEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, KineticVisualFieldsEventModel.QuestionCodes.Methodology));

            return descriptionList;
        }
    }
}