﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for MotilityEventViewManager
    /// </summary>
    public class MotilityEventViewManager : BaseEventViewManager<MotilityEventModel>
    {
        public override string CategoryCode
        {
            get { return "MotilityExMusc"; }
        }

        public MotilityEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override MotilityEventModel GetDefaultViewModel(MotilityEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.TestPerformed,
                MotilityEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.TestResult,
                MotilityEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.EventComment);

            viewModel.Pursuit = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Pursuit,
                MotilityEventModel.ChoiceCodes.Pursuit.NotDocumented);

            viewModel.Saccades = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Saccades,
                MotilityEventModel.ChoiceCodes.Saccades.NotDocumented);

            viewModel.Nystagmus = MapToSingleSelectQuestion(
                categoryQuestionList,
                MotilityEventModel.QuestionCodes.Nystagmus,
                MotilityEventModel.ChoiceCodes.Nystagmus.NotDocumented);

            viewModel.NystagmusOptions = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.NystagmusOptions);
            viewModel.Motility = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.Motility);
            viewModel.CranialNerve = MapToMultiSelectQuestion(categoryQuestionList, MotilityEventModel.QuestionCodes.CranialNerve);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override MotilityEventModel GetLoadedViewModel(MotilityEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.EventComment);
            viewModel.Pursuit = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Pursuit);
            viewModel.Saccades = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Saccades);
            viewModel.Nystagmus = MapToSingleSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Nystagmus);
            viewModel.NystagmusOptions = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.NystagmusOptions);
            viewModel.Motility = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.Motility);
            viewModel.CranialNerve = MapToMultiSelectQuestion(categoryQuestionList, responseList, MotilityEventModel.QuestionCodes.CranialNerve);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(MotilityEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                MotilityEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Pusuit
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Pursuit, referralId, encounterId,
                    lateralityCode));

                // Saccades
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Saccades, referralId, encounterId,
                    lateralityCode));

                // Nystagmus
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.Nystagmus, referralId, encounterId,
                    lateralityCode));

                if (viewModel.Nystagmus.IsAnswered &&
                    viewModel.Nystagmus.SelectedChoice.Code == MotilityEventModel.ChoiceCodes.Nystagmus.Present)
                {
                    // Nystagmus Options
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.NystagmusOptions, referralId,
                        encounterId, lateralityCode));
                }

                // Motility
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.Motility, referralId, encounterId,
                    lateralityCode));

                // Cranial Nerve
                responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(viewModel.CranialNerve, referralId,
                    encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event descriptions.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.Pursuit));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.Saccades));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.Nystagmus));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.Motility));
            descriptionList.Add(GetMultiSelectQuestionDescriptionLine(responses, MotilityEventModel.QuestionCodes.CranialNerve));

            return descriptionList;
        }
    }
}