﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Pupil Event View Manager
    /// </summary>
    public class PupilEventViewManager : BaseEventViewManager<PupilEventModel>
    {
        public override string CategoryCode
        {
            get { return "Pupil"; }
        }

        public PupilEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override PupilEventModel GetDefaultViewModel(PupilEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                PupilEventModel.QuestionCodes.TestPerformed,
                PupilEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                PupilEventModel.QuestionCodes.TestResult,
                PupilEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.EventComment);
            viewModel.Size = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Size);
            viewModel.AlterationInShape = MapToTextQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.AlterationInShape);
            viewModel.PerrlOrPerrla = MapToSingleSelectQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.PerrlOrPerrla);
            viewModel.Reactive = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Reactive);
            viewModel.Adie = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.Adie);
            viewModel.ArgyllRobertson = MapToYesNoQuestion(categoryQuestionList, PupilEventModel.QuestionCodes.ArgyllRobertson);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override PupilEventModel GetLoadedViewModel(PupilEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.EventComment);

            viewModel.Size = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Size);
            viewModel.AlterationInShape = MapToTextQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.AlterationInShape);
            viewModel.PerrlOrPerrla = MapToSingleSelectQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.PerrlOrPerrla);
            viewModel.Reactive = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Reactive);
            viewModel.Adie = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.Adie);
            viewModel.ArgyllRobertson = MapToYesNoQuestion(categoryQuestionList, responseList, PupilEventModel.QuestionCodes.ArgyllRobertson);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(PupilEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                PupilEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId, lateralityCode));

                // Size
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.Size, referralId, encounterId, lateralityCode));

                // Alteration In Shape
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.AlterationInShape, referralId, encounterId, lateralityCode));

                // PERRL or PERRLA
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.PerrlOrPerrla, referralId, encounterId, lateralityCode));

                if (viewModel.PerrlOrPerrla.IsAnswered)
                {
                    // Reactive
                    responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.Reactive, referralId, encounterId, lateralityCode));
                }

                // Adie's
                responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.Adie, referralId, encounterId, lateralityCode));

                // Argyll-Robertson
                responsesToBeAdded.Add(MapYesNoQuestionToResponse(viewModel.ArgyllRobertson, referralId, encounterId, lateralityCode));
            }

            return responsesToBeAdded;
        }

        /// <summary>Gets the event description.</summary>
        /// <param name="responses">The responses.</param>
        /// <returns></returns>
        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, PupilEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, PupilEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, PupilEventModel.QuestionCodes.Size));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, PupilEventModel.QuestionCodes.AlterationInShape));

            return descriptionList;
        }
    }
}