﻿using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Static Threshold Perimetry Event View Manager
    /// </summary>
    public class StaticThresholdPerimetryEventViewManager : BaseEventViewManager<StaticThresholdPerimetryEventModel>
    {
        public override string CategoryCode
        {
            get { return "StaticThresPer"; }
        }

        public StaticThresholdPerimetryEventViewManager(string currentUser) : base(currentUser)
        {
        }

        /// <summary>Gets the default view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <returns></returns>
        public override StaticThresholdPerimetryEventModel GetDefaultViewModel(StaticThresholdPerimetryEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                StaticThresholdPerimetryEventModel.QuestionCodes.TestPerformed,
                StaticThresholdPerimetryEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                StaticThresholdPerimetryEventModel.QuestionCodes.TestResult,
                StaticThresholdPerimetryEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.EventComment);
            viewModel.ReferenceScale = MapToSingleSelectQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.ReferenceScale);
            viewModel.MeanDeviation = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.MeanDeviation);
            viewModel.PatternStandardDeviation = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.PatternStandardDeviation);
            viewModel.FixationLoses = MapToTextQuestion(categoryQuestionList, StaticThresholdPerimetryEventModel.QuestionCodes.FixationLoses);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="categoryQuestionList">The category question list.</param>
        /// <param name="lateralityList">The laterality list.</param>
        /// <param name="responseList">The response list.</param>
        /// <returns></returns>
        public override StaticThresholdPerimetryEventModel GetLoadedViewModel(StaticThresholdPerimetryEventModel viewModel, IEnumerable<QUESTION> categoryQuestionList, IEnumerable<LATERALITY> lateralityList, IEnumerable<RESPONSE> responseList)
        {
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.TestPerformed);
            viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.TestResult);
            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.EventComment);
            viewModel.ReferenceScale = MapToSingleSelectQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.ReferenceScale);
            viewModel.MeanDeviation = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.MeanDeviation);
            viewModel.PatternStandardDeviation = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.PatternStandardDeviation);
            viewModel.FixationLoses = MapToTextQuestion(categoryQuestionList, responseList, StaticThresholdPerimetryEventModel.QuestionCodes.FixationLoses);

            return viewModel;
        }

        /// <summary>Gets the responses to be added.</summary>
        /// <param name="viewModel">The view model.</param>
        /// <param name="referralId">The referral identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <returns></returns>
        public override IEnumerable<RESPONSE> GetResponsesToBeAdded(StaticThresholdPerimetryEventModel viewModel, int referralId, int encounterId)
        {
            string lateralityCode = viewModel.SelectedLaterality != null ? viewModel.SelectedLaterality.Code : string.Empty;

            // Create Responses To Added
            List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

            // Test Performed
            responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestPerformed, referralId, encounterId, lateralityCode));

            // Event Comment
            responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.EventComment, referralId, encounterId, lateralityCode));

            if (viewModel.TestPerformed.IsAnswered &&
                StaticThresholdPerimetryEventModel.ShowTestDetailValues.Any(v => v == viewModel.TestPerformed.SelectedChoice.Code))
            {
                // Test Result
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.TestResult, referralId, encounterId,
                    lateralityCode));

                // Reference Scale
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(viewModel.ReferenceScale, referralId,
                    encounterId, lateralityCode));

                // Mean Deviation
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.MeanDeviation, referralId, encounterId,
                    lateralityCode));

                // Pattern Standard Deviation
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.PatternStandardDeviation, referralId,
                    encounterId, lateralityCode));

                // Fixation Loses
                responsesToBeAdded.Add(MapTextQuestionToResponse(viewModel.FixationLoses, referralId, encounterId,
                    lateralityCode));
            }

            return responsesToBeAdded;
        }

        public override IEnumerable<string> GetEventDescriptions(IEnumerable<RESPONSE> responses)
        {
            List<string> descriptionList = new List<string>();

            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.TestPerformed));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.TestResult));
            descriptionList.Add(GetSingleSelectQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.ReferenceScale));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.MeanDeviation));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.PatternStandardDeviation));
            descriptionList.Add(GetTextQuestionDescriptionLine(responses, StaticThresholdPerimetryEventModel.QuestionCodes.FixationLoses));

            return descriptionList;
        }
    }
}