﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using DataAccess;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.Reporting;

namespace Managers.Reporting
{
    /// <summary>
    /// The ReportViewManager class handles data access for the Reporting records in the database.
    /// </summary>
    public class ReportManager : BaseViewManager
    {
        public ReportManager(UserAccountManager currentUser)
            : base(currentUser)
        {
        }

        #region Member Variables

        private const string CONNECTION_NAME = "RegistryConnectionString";
        private const string GET_VISNS = "dbo.usp_GetVISNs";
        private const string GET_PATIENT_OVERVIEW_REPORT = "mevir.usp_GetPatientOverviewReport";
        private const string GET_ANNUAL_COMPLETION_REPORT = "mevir.usp_GetAnnualCompletionReport";
        private const string GET_UNIQUE_PATIENT_REPORT = "mevir.usp_GetUniquePatientReport";

        //private const string GET_REFERRALS_WITH_CRITERIA = "MEVIR.usp_GetReferralsByCriteria";
        //private const string GET_ENCOUNTERS_BY_REFERRALID = "MEVIR.usp_GetEncountersByReferralId";
        //private const string GET_FACILITIES_BY_VISN_ID = "MEVIR.usp_Get_Facilities_by_Visn_ID";
        //private const string GET_PATIENTSIDS_BY_REFERRAL_ID = "MEVIR.usp_Get_PatientSIDS_by_Referral_ID";
        //private const string GET_REFERRAL_DATE_BY_ENCOUNTER_ID = "MEVIR.usp_Get_Referral_Date_By_Encounter_ID";
        //private const string GET_CURRENT_USER_WORKLOAD_BY_ID = "MEVIR.usp_Get_Current_User_Workload_By_ID";

        //private const string GET_USERS_BY_ROLE_CODE = "MEVIR.usp_Get_Registry_Users_By_Role_Code";
        //private const string GET_USER_BY_USER_ID = "MEVIR.usp_Get_Registry_User_By_ID";
        //private const string GET_PROBLEM_LIST_BY_REFERRAL_ID = "MEVIR.usp_Get_Problem_List_Entries_by_Referral_ID";

        #endregion

        #region Static Methods

        /// <summary>
        /// GetVISNs: Returns a datatable of VISNs (ID, NAME)
        /// </summary>
        /// <returns></returns>
        public static DataTable GetVISNs()
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_VISNS, new object[]{});
        }

        /// <summary>
        /// Returns Patient Overview Data for passed VISNs and Years
        /// </summary>
        /// <param name="visnList"></param>
        /// <param name="years"></param>
        /// <returns></returns>
        public static DataTable GetPatientOverviewReport(string visnList, string years)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_PATIENT_OVERVIEW_REPORT, new object[] { visnList, years });
        }

        /// <summary>
        /// Returns Annual Completion Data for passed VISNs and Years
        /// </summary>
        /// <param name="visnList"></param>
        /// <param name="years"></param>
        /// <returns></returns>
        public static DataTable GetAnnualCompletionReport(string visnList, string years)
        {
            return SqlProvider.ExecuteSPDataTable(CONNECTION_NAME, GET_ANNUAL_COMPLETION_REPORT, new object[] { visnList, years });
        }

        /// <summary>
        /// Returns Unique Patient Data for Registry
        /// </summary>
        /// <returns></returns>
        public static DataSet GetUniquePatientReport()
        {
            return SqlProvider.ExecuteSPDataset(CONNECTION_NAME, GET_UNIQUE_PATIENT_REPORT, new object[] { });
        }

        /// <summary>
        /// Returns the total number of referrals for the VA EIDS registry.
        /// </summary>
        /// <returns></returns>
        public static int GetTotalNumberOfReferralsForRegistry()
        {
            using (var ctx = new MEVIREntities())
            {
                int regId = ctx.STD_REGISTRY.FirstOrDefault(R => R.CODE == "DVEIR").ID;
                return ctx.REFERRALs.Count(R => R.STD_REGISTRY_ID == regId);
            }
        }

        #endregion

        #region Report Views

        public UserEventLogReportViewModel GetUserEventLogReportViewModel()
        {
            IEnumerable<usp_GetAllRegistryUsers_Result> users = DataAccessManager.GetAllUsers();

            UserEventLogReportViewModel viewModel = new UserEventLogReportViewModel()
            {
                Users = users.ToDictionary(u => u.USERNAME, u => u.FULL_NAME)
            };

            return viewModel;
        }

        public UserEventLogReportViewModel GetUserEventLogReport(UserEventLogReportViewModel viewModel)
        {
            string username = viewModel.Username;
            DateTime dateFrom = viewModel.DateFrom;
            DateTime dateTo = viewModel.DateTo;
            int? referralId = viewModel.ReferralId == 0 ? (int?)null : viewModel.ReferralId;
            int? encounterId = viewModel.EncounterId == 0 ? (int?)null : viewModel.EncounterId;

            IEnumerable<usp_UserEventLog_Report_Result> userEventLogs =
                DataAccessManager.GetUserEventLogReport(username, dateFrom, dateTo, referralId, encounterId);

            viewModel.UserEventLogs = userEventLogs.Select(u => new UserEventLog()
            {
                Username = u.USERNAME,
                EventTime = u.CREATED,
                EventDescription = u.ACTIVITY,
                ReferralId = u.REFERRAL_ID,
                EncounterId = u.ENCOUNTER_ID,
                PatientName = u.PATIENT_NAME
            });

            return viewModel;
        }


        public DoDInterfaceTransferReportViewModel GetDoDInterfaceTransferReportViewModel()
        {
            return new DoDInterfaceTransferReportViewModel();
        }

        public DoDInterfaceTransferReportViewModel GetDoDInterfaceTransferReport(DoDInterfaceTransferReportViewModel viewModel)
        {
            DateTime dateFrom = viewModel.DateFrom;
            DateTime dateTo = viewModel.DateTo;

            IEnumerable<usp_DoDInterfaceTransfer_Report_Result> dodInterfaceTransfers =
                DataAccessManager.GetDoDInterfaceTransferReport(dateFrom, dateTo);

            viewModel.DoDInterfaceTransfers = dodInterfaceTransfers.Select(d => new DoDInterfaceTransfer()
            {
                PatientId = d.PatientSID ?? 0,
                FirstName = d.PatientFirstName,
                LastName = d.PatientLastName,
                Gender = string.IsNullOrEmpty(d.Gender) ? string.Empty : d.Gender == "M" ? "Male" : "Female",
                ReferralId = d.REFERRAL_ID,
                EncounterId = d.ER_ENCOUNTER_ID,
                EncounterDate = d.VISIT_DATE,
                TransferDate = d.DoD_DVEIR_SENT_DATE ?? DateTime.Now
            });

            return viewModel;
        }

        #endregion
    }
}