﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using Managers.Encounter;
using Telerik.Web.UI;
using BusinessManagerClasses;
using ViewModels.Encounter;

namespace Controls.Encounter
{
    public partial class DiagnosisControl : BaseWebControl
    {
        #region Member Variables

        /// <summary>Gets or sets the diagnosis view model.</summary>
        /// <value>The diagnosis view model.</value>
        public DiagnosisViewModel DiagnosisViewModel
        {
            get { return ViewState["DiagnosisViewModel"] as DiagnosisViewModel; }
            set { ViewState["DiagnosisViewModel"] = value; }
        }

        public IEnumerable<string> ReviewerCommentAjaxControls
        {
            get { return ViewState["ReviewerCommentAjaxControls"] as IEnumerable<string> ?? new string[0]; }
            set { ViewState["ReviewerCommentAjaxControls"] = value.ToList(); }
        }

        /// <summary>Occurs when [reviewer comment saved].</summary>
        public event EventHandler ReviewerCommentSaved;

        public event EventHandler DiagnosisSavedSuccessfully;

        /// <summary>Gets or sets the blind rehab indicator.</summary>
        /// <value>The blind rehab indicator.</value>
        private string BlindRehabIndicator
        {
            get
            {
                if (rbBlindRehabYes.Checked)
                {
                    return "Y";
                }
                else if (rbBlindRehabNo.Checked)
                {
                    return "N";
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if (value == "Y")
                {
                    rbBlindRehabYes.Checked = true;
                }
                else if (value == "N")
                {
                    rbBlindRehabNo.Checked = true;
                }
                else
                {
                    rbBlindRehabNA.Checked = true;
                }
            }
        }

        #endregion

        /// <summary>Default Page Load</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                BlindRehabIndicator = DiagnosisViewModel.BlindRehabIndicator;

                rbAddDiagnosisEntry.Text = "Add Diagnosis";
                rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";

                LoadControl();
                LoadReviewControl();

                ddlLaterality.DataSource = DiagnosisViewModel.AvailableLateralitiesOrdered;
                ddlLaterality.DataValueField = "Code";
                ddlLaterality.DataTextField = "DisplayText";
                ddlLaterality.DataBind();

                // Set Readonly
                if (DiagnosisViewModel.IsReadOnly)
                {
                    // Set buttons to readonly and disable the one that are not checked
                    rbBlindRehabYes.ReadOnly = true;
                    rbBlindRehabYes.Enabled = rbBlindRehabYes.Checked;

                    rbBlindRehabNo.ReadOnly = true;
                    rbBlindRehabNo.Enabled = rbBlindRehabNo.Checked;

                    rbBlindRehabNA.ReadOnly = true;
                    rbBlindRehabNA.Enabled = rbBlindRehabNA.Checked;

                    rgDiagnosis.MasterTableView.GetColumn("EditDiagnosisColumn").Display = false;
                    rgDiagnosis.MasterTableView.GetColumn("DeleteDiagnosisColumn").Display = false;
                }
            }
        }

        #region Grid Events

        /// <summary>Handles the ItemCommand event of the rgDiagnosis control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridCommandEventArgs"/> instance containing the event data.</param>
        protected void rgDiagnosis_ItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "EditDiagnosis")
            {
                int id;
                string summaryId = ((GridDataItem)e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    var summary = DiagnosisViewModel.DiagnosisSummaries.First(s => s.Id == id);
                    
                    // populate Add Panel with Edit ... 
                    rgDiagnosis.Visible = false;
                    pnlAddDiagnosisBtn.Visible = false;
                    pnlAddDiagnosisRegion.Visible = true;

                    hfEncounterDetailID.Value = summary.Id.ToString();
                    ddlLaterality.SelectedValue = summary.LateralityCode;
                    rtbDetailCode.Text = summary.Code;
                    rtbDescription.Text = summary.CodeDescription;

                    rbAddDiagnosisEntry.Text = "Update";
                    rbAddDiagnosisEntry.ToolTip = "Click to update the diagnosis entry";
                }
            }
            else if (e.CommandName == "DeleteDiagnosis")
            {
                int id;
                string summaryId = ((GridDataItem)e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    try
                    {
                        var viewManager = new DiagnosisViewManager(CurrentUser);
                        viewManager.DeleteDiagnosisDetail(DiagnosisViewModel.EncounterId, id);
                        DiagnosisViewModel = viewManager.GetDiagnosisSummaries(DiagnosisViewModel);

                        CreateLoggingRecord(DiagnosisViewModel.ReferralId, DiagnosisViewModel.EncounterId, DiagnosisViewModel.PatientName,
                            DVEIRUserActivityType.ENCOUNTER_DIAGNOSIS_SAVE, "Deleted Diagnosis");

                        LoadControl();
                    }
                    catch (Exception ex)
                    {
                        Logging.WriteLogToFile(ex.Message);
                    }
                }
            }
        }

        /// <summary>Handles the NeedDataSource event of the rgDiagnosis control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridNeedDataSourceEventArgs"/> instance containing the event data.</param>
        protected void rgDiagnosis_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            rgDiagnosis.DataSource = DiagnosisViewModel.DiagnosisSummaries;
        }

        #endregion

        #region GUI Events

        /// <summary>Add Button Clicked.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbAddDiagnosis_Click(object sender, EventArgs e)
        {
            rgDiagnosis.Visible = false;
            pnlAddDiagnosisBtn.Visible = false;
            pnlAddDiagnosisRegion.Visible = true;

            rbAddDiagnosisEntry.Text = "Add Diagnosis";
            rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";

            // clear values
            hfEncounterDetailID.Value = null;
            rtbDetailCode.Text = "";
            rtbDescription.Text = "";
            ddlLaterality.SelectedValue = DiagnosisViewModel.InitialLateralityCode;
        }

        /// <summary>Add Entry</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbAddDiagnosisEntry_Click(object sender, EventArgs e)
        {
            var summary = new DiagnosisSummary();

            int detailId = 0;
            int.TryParse(hfEncounterDetailID.Value, out detailId);

            summary.Id = detailId;
            summary.LateralityCode = ddlLaterality.SelectedValue;
            summary.Code = rtbDetailCode.Text;
            summary.CodeDescription = rtbDescription.Text;

            DiagnosisViewModel.DiagnosisSummary = summary;
            
            rgDiagnosis.Visible = true;
            pnlAddDiagnosisBtn.Visible = true;
            pnlAddDiagnosisRegion.Visible = false;
            
            var viewManager = new DiagnosisViewManager(CurrentUser);
            viewManager.SaveDiagnosisDetail(DiagnosisViewModel);

            // Added saved event
            if (DiagnosisSavedSuccessfully != null)
            {
                DiagnosisSavedSuccessfully(sender, e);
            }


            DiagnosisViewModel = viewManager.GetDiagnosisSummaries(DiagnosisViewModel);

            CreateLoggingRecord(DiagnosisViewModel.ReferralId, DiagnosisViewModel.EncounterId, DiagnosisViewModel.PatientName,
                            DVEIRUserActivityType.ENCOUNTER_DIAGNOSIS_SAVE, "Save Diagnosis");

            DiagnosisViewModel.DiagnosisSummary = null;

            LoadControl();
        }

        /// <summary>Cancel adding entry</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbCancelAddDiagnosisEntry_Click(object sender, EventArgs e)
        {
            rgDiagnosis.Visible = true;
            pnlAddDiagnosisBtn.Visible = true;

            // clear values
            rtbDetailCode.Text = "";
            rtbDescription.Text = "";
            hfEncounterDetailID.Value = null;

            rbAddDiagnosisEntry.Text = "Add Diagnosis";
            rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";

            pnlAddDiagnosisRegion.Visible = false;
        }

        /// <summary>
        /// Handles the OnReviewSavedSuccessfully event of the rccDiagnosisBlindRehab control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rccDiagnosisBlindRehab_OnReviewSavedSuccessfully(object sender, EventArgs e)
        {
            if (ReviewerCommentSaved != null)
            {
                ReviewerCommentSaved(sender, e);
            }
        }

        #endregion

        #region Public Methods

        /// <summary>Saves this instance.</summary>
        public void Save()
        {
            if (!DiagnosisViewModel.IsReadOnly)
            {
                DiagnosisViewModel.BlindRehabIndicator = BlindRehabIndicator;

                var viewManager = new DiagnosisViewManager(CurrentUser);
                viewManager.SaveDiagnosisView(DiagnosisViewModel);
            }
        }

        /// <summary>Shows the reviewer comment.</summary>
        /// <param name="questionCode">The question code.</param>
        public void ShowReviewerComment(string questionCode)
        {
            if (DiagnosisViewModel.AllowReviewerComment &&
                DiagnosisViewModel.BlindRehabReviewer.QuestionCode == questionCode)
            {
                rccDiagnosisBlindRehab.ReviewerCommentViewModel.ShowCommentText = true;
                rccDiagnosisBlindRehab.ReviewerCommentViewModel.SetFocus = true;
                rccDiagnosisBlindRehab.RefreshControl();
            }
        }

        #endregion

        #region Private Methods

        /// <summary>Loads the control.</summary>
        private void LoadControl()
        {
            pnlAddDiagnosisBtn.Visible = !DiagnosisViewModel.IsReadOnly;

            rgDiagnosis.DataSource = DiagnosisViewModel.DiagnosisSummaries;
            rgDiagnosis.DataBind();
        }

        /// <summary>Loads the review control.</summary>
        private void LoadReviewControl()
        {
            rccDiagnosisBlindRehab.ReviewerCommentViewModel = DiagnosisViewModel.BlindRehabReviewer;
            rccDiagnosisBlindRehab.Visible = DiagnosisViewModel.AllowReviewerComment;
            rccDiagnosisBlindRehab.AjaxUpdateControls = ReviewerCommentAjaxControls;
        }

        #endregion
    }
}