﻿using System;
using Managers.Encounter;
using Telerik.Web.UI;
using ViewModels.Encounter;
using BusinessManagerClasses;

namespace Controls.Encounter
{
    public partial class ProviderControl : BaseWebControl
    {
        #region Getter / Setters

        public ProviderViewModel ProviderViewModel 
        {
            get { return ViewState["ProviderViewModel"] as ProviderViewModel; }
            set { ViewState["ProviderViewModel"] = value; }
        }

        public event EventHandler ProviderSavedSuccessfully;

        #endregion

        /// <summary>Handles the Load event of the Page control.</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadData();
            }
        }

        #region Provider Grid Events

        /// <summary>
        /// Grid needs datasource
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rgProviders_NeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            rgProviders.DataSource = ProviderViewModel.ProviderSummaries;
        }

        /// <summary>
        /// Item Command Event on the Provider Grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rgProviders_ItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Delete")
            {
                int id;
                string summaryId = ((GridDataItem) e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    var viewManager = new ProviderViewManager(CurrentUser);
                    viewManager.DeleteEncounterProvider(ProviderViewModel, id);
                    ProviderViewModel = viewManager.RefreshProviderSummaries(ProviderViewModel);

                    CreateLoggingRecord(ProviderViewModel.ReferralId, ProviderViewModel.EncounterId, ProviderViewModel.PatientName,
                        DVEIRUserActivityType.ENCOUNTER_PROVIDER_SAVE, "Delete Provider");

                    LoadProviderSummary();
                }
            }

            if (e.CommandName == "MakePrimary")
            {
                int id;
                string summaryId = ((GridDataItem)e.Item).GetDataKeyValue("Id").ToString();

                if (int.TryParse(summaryId, out id))
                {
                    var viewManager = new ProviderViewManager(CurrentUser);
                    viewManager.SetEncounterProviderAsPrimary(ProviderViewModel, id);

                    if (ProviderSavedSuccessfully != null)
                    {
                        ProviderSavedSuccessfully(sender, e);
                    }

                    ProviderViewModel = viewManager.RefreshProviderSummaries(ProviderViewModel);

                    CreateLoggingRecord(ProviderViewModel.ReferralId, ProviderViewModel.EncounterId, ProviderViewModel.PatientName,
                        DVEIRUserActivityType.ENCOUNTER_PROVIDER_SAVE, "Make Primary Provider");

                    LoadProviderSummary();
                }
            }
        }

        #endregion

        #region GUI Events

        /// <summary>Add Provider Button Clicked ... show panel</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbAddProvider_Click(object sender, EventArgs e)
        {
            //rgProviders.Visible = false;
            pnlProviderSummary.Visible = false;
            pnlAddProviderRegion.Visible = true;

            // Default view
            rcbInstitution.SelectedValue = ProviderViewModel.CurrentFacility;
            rcbInstitution.Focus();
            GetProviderListByFacility(ProviderViewModel.CurrentFacility);
            cbIsPrimaryProvider.Checked = false;
            cbIsPrimaryProvider.Enabled = true;
        }

        protected void rcbInstitution_OnSelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
        {
            // Refresh Provider List
            GetProviderListByFacility(e.Value);
            rcbProviders.Focus();
        }

        /// <summary>Add Provider Entry Button Clicked</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbAddProviderEntry_Click(object sender, EventArgs e)
        {
            // Add entry to the LocalProviderList object ...
            var viewManager = new ProviderViewManager(CurrentUser);
            
            int providerId = Convert.ToInt32(rcbProviders.SelectedValue);
            bool isPrimaryProvider = cbIsPrimaryProvider.Checked;
            
            viewManager.SaveEncounterProvider(ProviderViewModel, providerId, isPrimaryProvider);

            if (ProviderSavedSuccessfully != null)
            {
                ProviderSavedSuccessfully(sender, e);
            }

            ProviderViewModel = viewManager.RefreshProviderSummaries(ProviderViewModel);

            CreateLoggingRecord(ProviderViewModel.ReferralId, ProviderViewModel.EncounterId, ProviderViewModel.PatientName,
                        DVEIRUserActivityType.ENCOUNTER_PROVIDER_SAVE, "Add Provider");

            pnlProviderSummary.Visible = true;
            pnlAddProviderRegion.Visible = false;
            LoadProviderSummary();
        }

        /// <summary>Cancel Adding Providers</summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void rbCancelAddProviderEntry_Click(object sender, EventArgs e)
        {
            pnlProviderSummary.Visible = true;
            pnlAddProviderRegion.Visible = false;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Private method to load data and initialize the form.
        /// </summary>
        private void LoadData()
        {
            LoadProviderSummary();

            rcbInstitution.DataSource = ProviderViewModel.Facilities;
            rcbInstitution.DataBind();
        }

        private void LoadProviderSummary()
        {
            phAddProvider.Visible = !ProviderViewModel.IsReadOnly;

            rgProviders.DataSource = ProviderViewModel.ProviderSummaries;
            rgProviders.DataBind();

            // Remove delete column if readonly
            rgProviders.MasterTableView.Columns[0].Visible = !ProviderViewModel.IsReadOnly;
        }

        private void GetProviderListByFacility(string selectedFacility)
        {
            var viewManager = new ProviderViewManager(CurrentUser);
            ProviderViewModel = viewManager.GetProvidersByFacility(ProviderViewModel, selectedFacility);

            // Query for providers based on Encounter data
            rcbProviders.DataSource = ProviderViewModel.Providers;
            rcbProviders.DataBind();
            rcbProviders.ClearSelection();
            rcbProviders.Text = string.Empty;
        }

        #endregion
    }
}