﻿using BusinessManagerClasses;
using Managers.Encounter;
using System;
using System.Linq;
using System.Text;
using System.Web.UI;
using Telerik.Web.UI;
using ViewModels.Encounter;
using EncounterStatusType = ViewModels.Encounter.EncounterStatusType;

public partial class Encounter : BasePage
{
    #region Member Variables

    /// <summary>Gets or sets the view model.</summary>
    /// <value>The view model.</value>
    protected EncounterViewModel ViewModel
    {
        get { return ViewState["ViewModel"] as EncounterViewModel; }
        set { ViewState["ViewModel"] = value; }
    }

    #endregion

    /// <summary>Default Page Load</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
    /// <exception cref="System.Exception">Missing Required Parameter - referral id
    /// or
    /// Missing Required Parameter - encounter id</exception>
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            #region Query String Parameter Parsing

            string qsReferralId = Request.QueryString.Get("refId");
            string qsEncounterId = Request.QueryString.Get("encId");
                
            int referralId;
            int encounterId;

            if (!int.TryParse(qsReferralId, out referralId))
            {
                throw new Exception("Missing Required Parameter - referral id");

            }

            if (!int.TryParse(qsEncounterId, out encounterId))
            {
                throw new Exception("Missing Required Parameter - encounter id");
            }

            #endregion

            EncounterViewManager viewManager = new EncounterViewManager(CurrentUser);
            ViewModel = viewManager.GetEncounterViewModel(referralId, encounterId);
                    
            if (ViewModel == null)
            {
                string errorMsg = 
                    string.Format("Encounter Not Found - referralId ({0}), encounterId ({1})", referralId, encounterId);

                throw new Exception(errorMsg);
            }

            // Log encounter was opened
            CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, DVEIRUserActivityType.ENCOUNTER_OPEN, string.Empty);

            LoadPageData();
            SetBreadCrumbText();
            ShowAvailableOperations();
        }
    }

    #region Public Overrides

    /// <summary>
    /// TopMenuName is the menu item in the top horizontal menu where this page falls under
    /// </summary>
    /// <returns></returns>
    public override string TopMenuName()
    {
        return "My Tasks";
    }

    /// <summary>MenuItemName is the value of the left menu link for this page</summary>
    /// <returns></returns>
    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    /// <summary>
    /// The list of capabilities that a user needs to access this page.  A single page may have more than one
    /// capability associated with it.
    /// </summary>
    /// <returns></returns>
    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region Events

    /// <summary>Fired on Toolbar option clicked.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="RadToolBarEventArgs"/> instance containing the event data.</param>
    protected void rtbAvailalbleOperations_ButtonClick(object sender, RadToolBarEventArgs e)
    {
        try
        {
            EncounterOperation encounterOperation;
            if (Enum.TryParse(e.Item.Value, out encounterOperation))
            {
                var viewManager = new EncounterViewManager(CurrentUser);
                switch (encounterOperation)
                {
                    case EncounterOperation.Save:
                        SaveEncounter();
                        UpdateStatusToInProgress();
                        CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, 
                            DVEIRUserActivityType.ENCOUNTER_SAVE, string.Empty);
                        break;
                    case EncounterOperation.ReadyForReview:
                        ViewModel.StatusType = EncounterStatusType.Review;
                        ViewModel = viewManager.SaveEncounterStatus(ViewModel);
                        CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, 
                            DVEIRUserActivityType.ENCOUNTER_SEND_FOR_REVIEW, string.Empty);

                        this.Redirect("ReferralEncounters.aspx?id=" + ViewModel.ReferralId);
                        break;
                    case EncounterOperation.ReturnToAbstractor:
                        ViewModel.StatusType = EncounterStatusType.SendBack;
                        ViewModel = viewManager.SaveEncounterStatus(ViewModel);
                        CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, 
                            DVEIRUserActivityType.ENCOUNTER_SEND_BACK_TO_ABSTRACTOR, string.Empty);

                        this.Redirect("ReferralEncounters.aspx?id=" + ViewModel.ReferralId);
                        break;
                    case EncounterOperation.SendToDVEIVR:
                        ViewModel.StatusType = EncounterStatusType.ReadyToSend;
                        ViewModel = viewManager.SaveEncounterStatus(ViewModel);
                        CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, 
                            DVEIRUserActivityType.ENCOUNTER_READY_TO_SEND_TO_DVEIR, string.Empty);

                        this.Redirect("ReferralEncounters.aspx?id=" + ViewModel.ReferralId);
                        break;
                    case EncounterOperation.Cancel:
                        ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
                        this.Redirect("ReferralEncounters.aspx?id=" + ViewModel.ReferralId);
                        break;
                }
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Processing Encounter Operation: " + ex.Message);
        }

        //toolBarItem.Value
    }

    /// <summary>Handles the OnReviewCommentListSelection event of the CommentListing control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="GridCommandEventArgs"/> instance containing the event data.</param>
    protected void CommentListing_OnReviewCommentListSelection(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            string quesCode = ((GridDataItem) e.Item).GetDataKeyValue("QuestionCode").ToString();
            string lateralityCode = ((GridDataItem) e.Item).GetDataKeyValue("LateralityCode").ToString();
            int anchorId = (int) ((GridDataItem) e.Item).GetDataKeyValue("AnchorId");
            long? groupingId = ((GridDataItem) e.Item).GetDataKeyValue("GroupingId") as long?;

            var tab = rtsTabStrip.Tabs.First(t => t.Value == anchorId.ToString());
            tab.Selected = true;
            tab.PageView.Selected = true;

            switch (anchorId)
            {
                case ReviewCommentAnchorType.ENCOUNTER_GENERAL_TAB:
                    // No review comments
                    break;
                case ReviewCommentAnchorType.ENCOUNTER_PROVIDER_TAB:
                    // No review comments
                    break;
                case ReviewCommentAnchorType.ENCOUNTER_CHIEF_COMPLAINT_TAB:
                    // Show Review Comment
                    complaintTabUserControl.ShowReviewerComment(quesCode);
                    break;
                case ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB:
                    // Show Review Comment
                    ExamTabUserControl.LoadEvent(quesCode, lateralityCode, groupingId ?? 0, true);
                    break;
                case ReviewCommentAnchorType.ENCOUNTER_DIAGNOSIS_TAB:
                    // Diagnosis Tab
                    diagnosisTabControl.ShowReviewerComment(quesCode);
                    break;
                case ReviewCommentAnchorType.ENCOUNTER_PROCEDURES_TAB:
                    // No review comments to show 
                    break;
            }
        }
    }

    /// <summary>Handles the OnEventSavedSuccessfully event of the ExamTabUserControl control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void TabUserControl_OnSavedSuccessfully(object sender, EventArgs e)
    {
        UpdateStatusToInProgress();
    }

    /// <summary>Handles the OnReviewerCommentSaved event of the TabUserControl control.</summary>
    /// <param name="sender">The source of the event.</param>
    /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
    protected void TabUserControl_OnReviewerCommentSaved(object sender, EventArgs e)
    {
        var viewManager = new EncounterViewManager(CurrentUser);
        ViewModel = viewManager.RefreshReviewerComments(ViewModel);

        CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, DVEIRUserActivityType.ENCOUNTER_REVIEWER_SAVE, string.Empty);

        CommentListing.ReviewCommentSummaries = ViewModel.GeneralView.ReviewCommentSummaries;
        CommentListing.RefreshReviewComments();
    }

    #endregion

    #region Private Methods

    /// <summary>Saves all the encounter tabs</summary>
    private void SaveEncounter()
    {
        if (CheckValidation())
        {
            // get general data
            EncounterInfo.Save();

            // get complaint text
            complaintTabUserControl.SaveChiefComplaint();

            // set diagnosis info
            diagnosisTabControl.Save();

            // Save Exam Tab
            ExamTabUserControl.Save();

            // Save History Tab
            historyTabControl.SaveHistory();
        }
    }

    /// <summary>Checks the validation.</summary>
    /// <returns></returns>
    private bool CheckValidation()
    {
        Validate();
        bool isPageValid = IsValid;

        if (!isPageValid)
        {
            string msg = "";

            foreach (IValidator validator in Validators)
            {
                if (!validator.IsValid)
                {
                    msg += @"<br />" + validator.ErrorMessage;
                }
            }

            // Create Js Alert
            string script = "<script type=\"text/javascript\">alert('" + msg + "');</script>";
            ClientScript.RegisterClientScriptBlock(typeof (Alert), "Errors", script);
        }

        return isPageValid;
    }

    /// <summary>Loads all page data.</summary>
    private void LoadPageData()
    {
        var reviewerCommentListing = new Control[] {pnlReviewCommentSummaries}.Select(c => c.ID);

        // Set General Tab Information
        EncounterInfo.GeneralViewModel = ViewModel.GeneralView;
        CommentListing.ReviewCommentSummaries = ViewModel.GeneralView.ReviewCommentSummaries;
        CommentListing.Visible = ViewModel.AllowReviewerComments;

        // Set Provider Tab Information
        ProviderTabControl.ProviderViewModel = ViewModel.ProviderView;

        // Set Chief Complaint Tab Information
        complaintTabUserControl.ChiefComplaintViewModel = ViewModel.ChiefComplaintView;
        complaintTabUserControl.ReviewerCommentAjaxControls = reviewerCommentListing;

        // Set Exam Tab Information
        ExamTabUserControl.ExamViewModel = ViewModel.ExamView;
        ExamTabUserControl.ReviewerCommentAjaxControls = reviewerCommentListing;

        // Set Diagnosis Tab Information
        diagnosisTabControl.DiagnosisViewModel = ViewModel.DiagnosisView;
        diagnosisTabControl.ReviewerCommentAjaxControls = reviewerCommentListing;

        // Set Procedure Tab Information
        procedureTabControl.ProcedureViewModel = ViewModel.ProceduresView;

        // Set History Tab Information
        historyTabControl.HistoryViewModel = ViewModel.HistoryView;
        historyTabControl.ReviewerCommentAjaxControls = reviewerCommentListing;
    }

    /// <summary>Shows the available operations.</summary>
    private void ShowAvailableOperations()
    {
        foreach (var item in rtbAvailalbleOperations.GetAllItems())
        {
            int itemValue = int.Parse(item.Value);
            item.Visible = ViewModel.AvaiilableEncounterOperations.Any(o => (int) o == itemValue);
        }
    }

    /// <summary>Sets the bread crumb text.</summary>
    private void SetBreadCrumbText()
    {
        StringBuilder breadCrumb = new StringBuilder();
        breadCrumb.Append("Referrals > ");
        breadCrumb.Append("<a href=\"ReferralEncounters.aspx?id=" + ViewModel.ReferralId +
                          "\" title=\"Click to go back to encounter listing for referral id: " + ViewModel.ReferralId +
                          "\" style=\"color:#FFFFFF;\">");
        breadCrumb.Append("Encounters for Referral (" + ViewModel.ReferralId + ")</a> > ");
        breadCrumb.Append("Encounter (" + ViewModel.EncounterId + ") - ");

        switch (ViewModel.StatusType)
        {
            case EncounterStatusType.New:
                breadCrumb.Append("New");
                break;
            case EncounterStatusType.InProc:
                breadCrumb.Append("In Process");
                break;
            case EncounterStatusType.Review:
                breadCrumb.Append("Ready for Review");
                break;
            case EncounterStatusType.SendBack:
                breadCrumb.Append("Send Back to Abstractor");
                break;
            case EncounterStatusType.ReadyToSend:
                breadCrumb.Append("Ready to Send to DVEIVR");
                break;
            case EncounterStatusType.Completed:
                breadCrumb.Append("Completed");
                break;
        }

        ((RegistryNoLeftMenu) Page.Master).SetBreadCrumbLabel(breadCrumb.ToString());
    }

    /// <summary>Updates the status to in progress.</summary>
    private void UpdateStatusToInProgress()
    {
        if (ViewModel.StatusType == EncounterStatusType.New)
        {
            ViewModel.StatusType = EncounterStatusType.InProc;
            var viewManager = new EncounterViewManager(CurrentUser);

            ViewModel = viewManager.SaveEncounterStatus(ViewModel);
            ShowAvailableOperations();
            SetBreadCrumbText();

            // Force Update on the pnl
            var ajaxManager = RadAjaxManager.GetCurrent(Page);
            ajaxManager.ResponseScripts.Add("__doPostBack('ForcePostback','')");

            // Log Status Change
            CreateLoggingRecord(ViewModel.ReferralId, ViewModel.EncounterId, ViewModel.PatientName, DVEIRUserActivityType.ENCOUNTER_SET_TO_IN_PROCESS, string.Empty);
        }
    }

    #endregion
}