﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class DepthPerceptionEventViewManager : BaseEventViewManager<DepthPerceptionEventModel>
    {
        public override DepthPerceptionEventModel GetDefaultViewModel()
        {
            var viewModel = new DepthPerceptionEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                DepthPerceptionEventModel.QuestionCodes.TestPerformed,
                DepthPerceptionEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                DepthPerceptionEventModel.QuestionCodes.TestResult,
                DepthPerceptionEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.EventComment);
            viewModel.RandotFly = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.RandotFly);
            viewModel.VerhoeffSteropter = MapToTextQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VerhoeffSteropter);
            viewModel.RandotStereoTest = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.RandotStereoTest);
            viewModel.TitmusStereoTest = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.TitmusStereoTest);
            viewModel.VectographicProjectorSlides = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VectographicProjectorSlides);
            viewModel.VisionTesterDepth = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.VisionTesterDepth);
            viewModel.BorishVectographicNearpoint = MapToSingleSelectQuestion(categoryQuestionList, DepthPerceptionEventModel.QuestionCodes.BorishVectographicNearpoint);

            return viewModel;
        }

        public override DepthPerceptionEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new DepthPerceptionEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TestResult);

                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.EventComment);
                viewModel.RandotFly = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.RandotFly);
                viewModel.VerhoeffSteropter = MapToTextQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VerhoeffSteropter);
                viewModel.RandotStereoTest = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.RandotStereoTest);
                viewModel.TitmusStereoTest = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.TitmusStereoTest);
                viewModel.VectographicProjectorSlides = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VectographicProjectorSlides);
                viewModel.VisionTesterDepth = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.VisionTesterDepth);
                viewModel.BorishVectographicNearpoint = MapToSingleSelectQuestion(categoryQuestionList, responseList, DepthPerceptionEventModel.QuestionCodes.BorishVectographicNearpoint);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(DepthPerceptionEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    DepthPerceptionEventModel.ShowTestDetailValues.Any(v => v== result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Randot Stereo Test
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.RandotStereoTest, patientId,
                        encounterId, lateralityCode));

                    // Randot Fly
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.RandotFly, patientId, encounterId,
                        lateralityCode));

                    // Titmus Stereo Test 
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TitmusStereoTest, patientId,
                        encounterId, lateralityCode));

                    // Vectographic Projector Slides
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.VectographicProjectorSlides,
                        patientId, encounterId, lateralityCode));

                    // Vision Tester Depth Perception Slide
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.VisionTesterDepth, patientId,
                        encounterId, lateralityCode));

                    // Borish Vectographic Nearpoint Card
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.BorishVectographicNearpoint,
                        patientId, encounterId, lateralityCode));

                    // Verhoeff Steropter
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.VerhoeffSteropter, patientId, encounterId,
                        lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}