﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Intraocular Pressure Event View Manager
    /// </summary>
    public class IntraocularPressureEventViewManager : BaseEventViewManager<IntraocularPressureEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The intraocular pressure view model</returns>
        public override IntraocularPressureEventModel GetDefaultViewModel()
        {
            var viewModel = new IntraocularPressureEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                IntraocularPressureEventModel.QuestionCodes.TestPerformed,
                IntraocularPressureEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                IntraocularPressureEventModel.QuestionCodes.TestResult,
                IntraocularPressureEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.EventComment);
            viewModel.Method = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Method);
            viewModel.MethodComment = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.MethodComment);
            viewModel.Number = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Number);
            viewModel.Time = MapToTextQuestion(categoryQuestionList, IntraocularPressureEventModel.QuestionCodes.Time);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The intraocular pressure view model</returns>
        public override IntraocularPressureEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new IntraocularPressureEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = MapToSingleSelectQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = MapToSingleSelectQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.TestResult);
                viewModel.EventComment = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.EventComment);

                viewModel.Method = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Method);
                viewModel.MethodComment = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.MethodComment);
                viewModel.Number = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Number);
                viewModel.Time = MapToTextQuestion(categoryQuestionList, responseList, IntraocularPressureEventModel.QuestionCodes.Time);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(IntraocularPressureEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    IntraocularPressureEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Method
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.Method, patientId, encounterId,
                        lateralityCode));

                    // Method Comment
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.MethodComment, patientId, encounterId,
                        lateralityCode));

                    // Number
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.Number, patientId, encounterId,
                        lateralityCode));

                    // Time
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.Time, patientId, encounterId, lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}