﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class KineticVisualFieldsEventViewManager : BaseEventViewManager<KineticVisualFieldsEventModel>
    {
        public override KineticVisualFieldsEventModel GetDefaultViewModel()
        {
            var viewModel = new KineticVisualFieldsEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality = 
                MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                KineticVisualFieldsEventModel.QuestionCodes.TestPerformed,
                KineticVisualFieldsEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                KineticVisualFieldsEventModel.QuestionCodes.TestResult,
                KineticVisualFieldsEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, KineticVisualFieldsEventModel.QuestionCodes.EventComment);
            viewModel.Methodology = MapToMultiSelectQuestion(categoryQuestionList, KineticVisualFieldsEventModel.QuestionCodes.Methodology);

            viewModel.Results =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.Methodology, KineticVisualFieldsEventModel.QuestionCodes.ResultSuffix);

            return viewModel;
        }

        public override KineticVisualFieldsEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new KineticVisualFieldsEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed = 
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.TestPerformed);
                
                viewModel.TestResult =
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.TestResult);
                
                viewModel.EventComment =
                    MapToTextQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.EventComment);

                viewModel.Methodology = 
                    MapToMultiSelectQuestion(categoryQuestionList, responseList, KineticVisualFieldsEventModel.QuestionCodes.Methodology);
                
                viewModel.Results =
                    MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.Methodology, KineticVisualFieldsEventModel.QuestionCodes.ResultSuffix);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(KineticVisualFieldsEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    KineticVisualFieldsEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId,
                        lateralityCode));

                    // Methodology
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.Methodology, patientId,
                        encounterId, lateralityCode));

                    // Results
                    responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(result.Results,
                        result.Methodology, patientId, encounterId, lateralityCode));
                }
                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}