﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for Symptoms Event View Manager
    /// </summary>
    public class VisualFunctioningEventViewManager : BaseEventViewManager<VisualFunctioningEventModel>
    {
        /// <summary>Gets the default view model.</summary>
        /// <returns>The visual functioning view model</returns>
        public override VisualFunctioningEventModel GetDefaultViewModel()
        {
            var viewModel = new VisualFunctioningEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);
            viewModel.Questionnaire = MapToSingleSelectQuestion(categoryQuestionList, VisualFunctioningEventModel.QuestionCodes.Questionnaire);
            viewModel.Score = MapToTextQuestion(categoryQuestionList, VisualFunctioningEventModel.QuestionCodes.Score);

            return viewModel;
        }

        /// <summary>Gets the loaded view model.</summary>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        /// <param name="lateralityCode">The laterality code.</param>
        /// <returns>The visual functioningy view model</returns>
        public override VisualFunctioningEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new VisualFunctioningEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.Questionnaire = MapToSingleSelectQuestion(categoryQuestionList, responseList, VisualFunctioningEventModel.QuestionCodes.Questionnaire);
                viewModel.Score = MapToTextQuestion(categoryQuestionList, responseList, VisualFunctioningEventModel.QuestionCodes.Score);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        /// <summary>Saves the specified result.</summary>
        /// <param name="result">The result.</param>
        /// <param name="patientId">The patient identifier.</param>
        /// <param name="encounterId">The encounter identifier.</param>
        public override void Save(VisualFunctioningEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();
                
                // Questionnaire
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.Questionnaire, patientId, encounterId, lateralityCode));

                // Score
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.Score, patientId, encounterId, lateralityCode));

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}