﻿using System;
using System.Linq;
using System.Web.UI;
using DataAccess;
using Factories;
using Managers.Encounter;
using Managers.Exam;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabUserControl : UserControl
    {
        protected const string ExamTabName = "Exam Tab";

        public ER_ENCOUNTER_MOD Encounter { get; set; }

        public UserAccountManager CurrentUser { get; set; }

        public event SaveEvent EventSavedSuccessfully;
        public delegate void SaveEvent();

        public bool IsEditable { get; set; }

        private string LastLoadedEventCode
        {
            get
            {
                return ViewState["LastLoadedEvent"] as string;
            }
            set
            {
                ViewState["LastLoadedEvent"] = value;
            }
        }

        private bool CurrentEventItemInitalized
        {
            get
            {
                return ViewState["CurrentEventItemInitalized"] as bool? ?? false;
            }
            set
            {
                ViewState["CurrentEventItemInitalized"] = value;
            }
        }

        private ExamViewModel ExamViewModel
        {
            get { return ViewState["ExamViewModel"] as ExamViewModel; }
            set { ViewState["ExamViewModel"] = value; }
        }

        public bool Save()
        {
            return SaveCurrentEvent();
        }

        public void LoadEvent(string categoryCode, string lateralityCode, bool setReviewFocus)
        {
            LoadUserControl(categoryCode, lateralityCode);
            LoadReviewCommentControl(categoryCode, lateralityCode, setReviewFocus);

            ddlEventTypes.SelectedValue = categoryCode;
            ddlEventTypes.Enabled = false;

            // Switch to update/cancel buttons
            btnSaveEvent.Visible = false;
            btnUpdateEvent.Visible = IsEditable;
            btnCancelUpdate.Visible = true;
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                LoadViewModel();
                DataBindEventSummaries();
                DataBindEventCategories();
            }

            LoadUserControl(!CurrentEventItemInitalized);
        }

        protected void ddlEventTypes_OnSelectedIndexChanged(object sender, EventArgs e)
        {
            // Set the new event code
            LastLoadedEventCode = ddlEventTypes.SelectedValue;
            LoadUserControl(true);
        }

        protected void btnSaveEvent_OnClick(object sender, EventArgs e)
        {
            SaveCurrentEvent();
        }

        protected void btnCancelUpdate_OnClick(object sender, EventArgs e)
        {
            // Simply Clear The Event Category
            ClearEventCategory();
        }

        protected void gvEventList_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            string[] commandArgs = e.CommandArgument.ToString().Split(new char[] { ',' });

            if (e.CommandName == "EditEvent")
            {
                string categoryCode = commandArgs[0];
                string lateralityCode = commandArgs[1];
                LoadEvent(categoryCode, lateralityCode, false);
            }

            if (e.CommandName == "DeleteEvent")
            {
                string categoryCode = commandArgs[0];
                string lateralityCode = commandArgs[1];
                ExamEventFactoryResult factoryResult = ExamEventFactory.GetExamEventFactoryResult(categoryCode);

                // Get current factory and delete selected Event
                if (factoryResult != null)
                {
                    factoryResult.EventViewManager.Delete(Encounter.REFERRAL_ID, Encounter.ER_ENCOUNTER_ID, lateralityCode, categoryCode);
                }

                // Refresh Model and Summary Grid
                LoadViewModel();
                DataBindEventSummaries();
            }
        }

        protected void gvEventList_OnNeedDataSource(object sender, GridNeedDataSourceEventArgs e)
        {
            gvEventList.DataSource = ExamViewModel.EventSummariesOrdered;
        }

        private void DataBindEventCategories()
        {
            ddlEventTypes.DataSource = ExamViewModel.EventCategoriesOrdered;
            ddlEventTypes.DataValueField = "Code";
            ddlEventTypes.DataTextField = "Name";
            ddlEventTypes.DataBind();
        }

        private void DataBindEventSummaries()
        {
            gvEventList.DataSource = ExamViewModel.EventSummariesOrdered;
            gvEventList.DataBind();
        }

        private void LoadViewModel()
        {
            ExamViewManager viewManager = new ExamViewManager();
            ExamViewModel = viewManager.GetExamViewModel(Encounter.REFERRAL_ID, Encounter.ER_ENCOUNTER_ID);
        }

        private void LoadReviewCommentControl(string categoryCode, string lateralityCode, bool setReviewFocus)
        {
            var questionCategory = ExamViewModel.EventCategories.FirstOrDefault(c => c.Code == categoryCode);

            if (questionCategory != null)
            {
                rccExamEvent.EncounterId = Encounter.ER_ENCOUNTER_ID;
                rccExamEvent.QuestionCode = questionCategory.Code;
                rccExamEvent.LateralityCode = lateralityCode;
                rccExamEvent.AnchorId = ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB;
                rccExamEvent.FriendlyLocationName = ExamTabName + " > " + questionCategory.Name;
                rccExamEvent.IsCommentEditable = true;
                //rccChiefComplaint.SetFocusOnLoad
                rccExamEvent.CurrentUser = CurrentUser;

                rccExamEvent.EncounterStatusID = Encounter.EncounterStatusID ?? 0;

                rccExamEvent.ShowCommentsOnLoad = setReviewFocus;
                rccExamEvent.SetFocusOnLoad = setReviewFocus;
                rccExamEvent.IsSelectedFromReviewGrid = setReviewFocus;

                rccExamEvent.SetControlState();
                rccExamEvent.LoadControlsFromObject();
            }
        }

        private void LoadUserControl(bool initializeControl)
        {
            string categoryCode = LastLoadedEventCode;
            LoadEventCategoryUserControl(initializeControl, true, categoryCode, string.Empty);
        }

        private void LoadUserControl(string categoryCode, string lateralityCode)
        {
            LoadEventCategoryUserControl(true, false, categoryCode, lateralityCode);
        }

        private void LoadEventCategoryUserControl(bool initializeControl, bool loadDefaultView, string categoryCode, string lateralityCode)
        {
            phEventCategory.Controls.Clear();
            pnlEyeExamEventCategory.GroupingText = string.Empty;

            ExamEventFactoryResult factoryResult = ExamEventFactory.GetExamEventFactoryResult(categoryCode);

            if (factoryResult != null)
            {
                var uc = (UserControl)LoadControl(factoryResult.EventControlLocation);
                uc.ID = factoryResult.EventControlId;


                if (initializeControl)
                {
                    BaseEventModel eventViewModel =
                        loadDefaultView
                        ? factoryResult.EventViewManager.GetDefaultViewModel()
                        : factoryResult.EventViewManager.GetLoadedViewModel(ExamViewModel.PatientId, ExamViewModel.EncounterId, lateralityCode);

                    // Set the view model and validation group
                    var baseEventItem = uc as IBaseEventItem;
                    if (baseEventItem != null)
                    {
                        baseEventItem.SetViewModel(eventViewModel);
                        baseEventItem.SetValidationGroup(btnSaveEvent.ValidationGroup);
                    }

                    CurrentEventItemInitalized = true;

                    // Get Section Name
                    var eventCategory = ExamViewModel.EventCategories.First(e => e.Code == categoryCode);
                    pnlEyeExamEventCategory.GroupingText = eventCategory.Name;
                }

                // Add User Control
                phEventCategory.Controls.Add(uc);
            }
            else
            {
                CurrentEventItemInitalized = false;
            }

            LastLoadedEventCode = categoryCode;
        }

        private bool SaveCurrentEvent()
        {
            bool saveSuccessful = false;
            if (IsEditable && Page.IsValid)
            {
                string eventCode = LastLoadedEventCode;

                ExamEventFactoryResult factoryResult = ExamEventFactory.GetExamEventFactoryResult(eventCode);
                // Get current factory, update view model, save model to database
                if (factoryResult != null)
                {
                    IBaseEventItem eventItem = (IBaseEventItem)phEventCategory.FindControl(factoryResult.EventControlId);
                    BaseEventModel viewModel = eventItem.SaveItem();
                    factoryResult.EventViewManager.Save(viewModel, ExamViewModel.PatientId, ExamViewModel.EncounterId);
                }

                ClearEventCategory();

                if (EventSavedSuccessfully != null)
                {
                    EventSavedSuccessfully();
                }
                saveSuccessful = true;
            }

            return saveSuccessful; // Need some validation handled
        }

        private void ClearEventCategory()
        {
            // Clear current view
            LastLoadedEventCode = string.Empty;

            // Refresh Model and Summary Grid
            LoadViewModel();
            DataBindEventSummaries();

            // Event Item should be cleared and enabled
            ddlEventTypes.ClearSelection();
            ddlEventTypes.Enabled = true;
            LoadUserControl(false);

            btnSaveEvent.Visible = IsEditable;
            btnUpdateEvent.Visible = false;
            btnCancelUpdate.Visible = false;
        }
    }
}